// DragUtils.h: interface for the CDragUtils class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DRAGUTILS_H__E6C77B92_5C1E_4F7B_8367_6349A36C01D5__INCLUDED_)
#define AFX_DRAGUTILS_H__E6C77B92_5C1E_4F7B_8367_6349A36C01D5__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "JunctionUtils.h"

class CDrawingObject;
class CTinyCadDoc;

class CDragUtils  
{
protected:
	struct dragWire
	{
		BOOL			m_MoveA;
		BOOL			m_MoveB;
		CDrawingObject* m_Object;

		CDrawingObject* m_Attached_LineA;
		double			m_Distance_AlongA;

		CDrawingObject* m_Attached_LineB;
		double			m_Distance_AlongB;

		BOOL			m_Done_A;
		BOOL			m_Done_B;

		BOOL			m_Lock_Horiz;
		BOOL			m_Lock_Vert;

		dragWire( CDrawingObject *o, BOOL MoveA = FALSE, BOOL MoveB = FALSE)
		{
			m_Object = o;
			m_MoveA = MoveA;
			m_MoveB = MoveB;
			m_Attached_LineA = NULL;
			m_Distance_AlongA = 0.0;
			m_Attached_LineB = NULL;
			m_Distance_AlongB = 0.0;
			m_Done_A = FALSE;
			m_Done_B = FALSE;
			m_Lock_Horiz = o->a.y == o->b.y;
			m_Lock_Vert = o->a.x == o->b.x;
		}

		void Lock()
		{
			if (m_Lock_Horiz)
			{
				if (!m_Done_B)
				{
					m_Object->b.y = m_Object->a.y;
					m_Done_B = TRUE;
				}
				if (!m_Done_A)
				{
					m_Object->a.y = m_Object->b.y;
					m_Done_A = TRUE;
				}
			}
			

			if (m_Lock_Vert)
			{
				if (!m_Done_B)
				{
					m_Object->b.x = m_Object->a.x;
					m_Done_B = TRUE;
				}
				if (!m_Done_A)
				{
					m_Object->a.x = m_Object->b.x;
					m_Done_A = TRUE;
				}
			}
		}

		bool operator<( const dragWire &b ) const
		{
			return m_Object < b.m_Object;
		}
	};

	typedef std::set<dragWire> wireCollection;
	wireCollection	m_draggedWires;

	typedef std::set<CDrawingObject *> discardCollection;
	discardCollection	m_discards;


	void AddWiresAtPoint( CPoint p );
	void AddWireToCollection( dragWire n );
	void AddAttachedObjects( CDrawingObject *wire, BOOL wire_move_a, BOOL wire_move_b );
	void MoveAttachedObjects( CDrawingObject *wire );
	void DisplayDraggedWires();

	// Our current design that this object manipulates
	CTinyCadDoc	*m_pDesign;					

	// Have we started?
	BOOL m_started;

	// The stored begin data
	CPoint m_a;
	CPoint m_b;

	void Begin();
	void Clear();
	void Merge();
	void Clean();

	// Our junction utils
	CJunctionUtils	m_j;

public:
	CDragUtils& operator=( const CDragUtils &o )
	{
		// Don't do this!
		ASSERT( FALSE );
		return *this;
	}

	CDragUtils( CTinyCadDoc* pDesign );
	virtual ~CDragUtils();

	void Begin( CPoint a, CPoint b );
	void Drag( CPoint r );
	void Move( CPoint r );
	void End( bool no_clean );
	void MergeLinePoint( CDrawingObject *p );

	BOOL Started() { return m_started; }
};

#endif // !defined(AFX_DRAGUTILS_H__E6C77B92_5C1E_4F7B_8367_6349A36C01D5__INCLUDED_)
