/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "TinyCadView.h"
#include "diag.h"
#include "colour.h"


// This defines the block edit functions

// The rectangle area slightly re-written
#define theArea CRect(a.x,a.y,b.x,b.y)




////// The block import object //////


CDrawBlockImport::CDrawBlockImport(CTinyCadDoc *pDesign)
: CDrawingObject( pDesign )
{
  m_segment=1;
}

void CDrawBlockImport::Import( BOOL file_load )
{
  // Has the import data been selected yet?
	if (file_load)
	{
		m_pDesign->Import();
	}

  
  if (m_pDesign->IsSelected()) 
  {
	a=b=CPoint(0,0);

	selectIterator it = m_pDesign->GetSelectBegin();
	while ( it != m_pDesign->GetSelectEnd() ) 
	{
		CDrawingObject *obj=*it;

		b.x = max(b.x,max(obj->a.x,obj->b.x));
		b.y = max(b.y,max(obj->a.y,obj->b.y));
		obj->Display();

		++ it;
	}

	m_segment = 0;
  }

  m_pDesign->PostPaste();

  m_pDesign->MarkSelectChangeForUndo();
}


void CDrawBlockImport::EndEdit()
{
  RButtonDown(CPoint(0,0), CPoint(0,0));
}


void CDrawBlockImport::LButtonDown(CPoint p, CPoint)
{
  // New undo level for each placement...
  m_pDesign->BeginNewChangeSet();


  if (!m_segment)
  {
	m_pDesign->UnSelect();
	m_segment=1;

	// Now de-select back to the edit item
	m_pDesign->SelectObject( new CDrawEditItem( m_pDesign ) );
  }
}


void CDrawBlockImport::Move(CPoint p, CPoint no_snap_p)
{
  if (!m_segment) {
  	CPoint r;

  	r.x=p.x-b.x;
  	r.y=p.y-b.y;

	if (r.x!=0 || r.y!=0) 
	{
		m_pDesign->SelectMove(r);
		b=p;
	}
  }
}



BOOL CDrawBlockImport::RButtonDown(CPoint p, CPoint s)
{
  // If necessary remove the line from the screen
  if (!m_segment) {
	m_pDesign->SelectDelete();
	m_pDesign->Invalidate();
	m_segment=1;
	a = b;
  }

  return FALSE;
}



