/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "TinyCadView.h"
#include "diag.h"
#include "colour.h"


// This defines the block edit functions

// The rectangle area slightly re-written
#define theArea CRect(a.x,a.y,b.x,b.y)




////// The block move object //////

void CDrawBlockMove::Paint(Context &dc,paint_options options)
{

  dc.SelectBrush();
  dc.SelectPen(PS_DOT,1,cBLOCK);
	dc.SetROP2(R2_COPYPEN);

  dc.Rectangle(theArea);
}


CDrawBlockMove::CDrawBlockMove(CTinyCadDoc *pDesign)
: CDrawingObject( pDesign )
{
  m_segment=1;
  placed=0;
  a=b=CPoint(0,0);
}


int CDrawBlockMove::SetCursor( CPoint p )
{
	return placed ? 11 : 12;
}

void CDrawBlockMove::EndEdit()
{

  // Cancel the place operation
  if (placed)
	Move(OldPos,OldPos);

  if (!m_segment | placed) {
	m_pDesign->UnSelect();
	Display();
  }

  m_pDesign->Invalidate();
}

void CDrawBlockMove::LButtonDown(CPoint p, CPoint)
{
  // New undo level for each placement...
  m_pDesign->BeginNewChangeSet();


	
  if (placed) 
  {
	Display();
	m_pDesign->SelectUndoMove(CPoint(OldPos.x-b.x,OldPos.y-b.y));
	m_pDesign->UnSelect();
	m_segment=1;
 	placed=0;
	a = b;
	m_pDesign->SetModifiedFlag( TRUE );
	m_pDesign->Invalidate();
  } else if (m_segment) {
	a=p;
	b=p;
	m_segment=0;
	placed=0;
	Display();
  } else {
	Display();
	b=p;
	placed=1;
	m_segment=0;
	// Now select all the objects in this rectangle
	m_pDesign->Select(a,b);
	m_pDesign->MarkSelectChangeForUndo();
	OldPos = b;
	Display();
  }
}


void CDrawBlockMove::Move(CPoint p, CPoint no_snap_p)
{
  if (placed) {
  	CPoint r;

  	r.x=p.x-b.x;
  	r.y=p.y-b.y;

	if (r.x!=0 || r.y!=0) {
		Display();
		m_pDesign->SelectMove(r);
		a.x += r.x;
		a.y += r.y;
		b=p;
		Display();
	}
  } else if (!m_segment) {
	Display();
	b=p;
	Display();
  }

}



BOOL CDrawBlockMove::RButtonDown(CPoint p, CPoint s)
{
  BOOL r = m_pDesign->IsSelected();

  // Cancel the place operation
  if (placed)
	Move(OldPos,OldPos);

  // If necessary remove the line from the screen
  if (!m_segment | placed) {
	Display();
	m_pDesign->UnSelect();
	m_segment=1;
 	placed=0;
	a = b;
  }

  return r;
}


