/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include "tinycad.h"
#include "LibraryDoc.h"
#include "TinyCadSymbolDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLibraryDoc

IMPLEMENT_DYNCREATE(CLibraryDoc, CDocument)

CLibraryDoc::CLibraryDoc()
{
	m_selected = -1;
	m_pLibrary = NULL;
}

BOOL CLibraryDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;
	return TRUE;
}

CLibraryDoc::~CLibraryDoc()
{
}


BEGIN_MESSAGE_MAP(CLibraryDoc, CDocument)
	//{{AFX_MSG_MAP(CLibraryDoc)
	ON_COMMAND(ID_FILE_EXPORTLIBRARY, OnFileExportlibrary)
	ON_COMMAND(ID_FILE_IMPORTLIBRARY, OnFileImportlibrary)
	ON_COMMAND(ID_CONTEXT_EXPORTSYMBOL, OnContextExportsymbol)
	ON_UPDATE_COMMAND_UI(ID_CONTEXT_EXPORTSYMBOL, OnUpdateContextExportsymbol)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLibraryDoc diagnostics

#ifdef _DEBUG
void CLibraryDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CLibraryDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CLibraryDoc serialization

void CLibraryDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CLibraryDoc commands

void CLibraryDoc::setLibrary(CLibraryStore *pLibrary)
{
	// Keep a track of this library
	m_pLibrary = pLibrary;

	// Now set our name accordingly
	SetTitle(pLibrary->m_name);


	// Make a map of the library names, so they are sorted in
	// name order in our list...
	m_SymbolMap.clear();
	for ( CLibraryStore::symbolCollection::iterator i = m_pLibrary->m_Symbols.begin(); i != m_pLibrary->m_Symbols.end(); i++ )
	{
		int id = i->first;
		CLibraryStoreNameSet &name_set = i->second ;
		CSymbolRecord &r = name_set.GetRecord( 0 );

		m_SymbolMap.insert( std::pair<CString,int>(r.name, id) );
	}


	// Now make a list of the heights of each of the symbols
	m_heights.clear();

	// We are no longer selecting an object
	m_selected = -1;

	// .. and make a check of the symbol changes
	SetModifiedFlag( FALSE );
}

void CLibraryDoc::ResetSymbols()
{
	// Erase everything
	m_heights.clear();
	m_selected = -1;

	// Now re-load the symbols
	setLibrary( m_pLibrary );

	// Now invalidate this window
	UpdateAllViews( NULL );
}


// Is this document editing a library?
bool CLibraryDoc::IsLibInUse( CLibraryStore *lib )
{
	return lib == m_pLibrary;
}


CSize CLibraryDoc::GetDocSize()
{
	if (m_heights.size() > 0)
	{
		return CSize(0,*(m_heights.rbegin()));
	}
	else
	{
		return CSize(0,1024);
	}
}


CLibraryStoreNameSet &CLibraryDoc::getNameSet( const idCollection::iterator &it )
{
	return m_pLibrary->m_Symbols[ it->second ];
}


CLibraryStore::symbolCollection::iterator CLibraryDoc::getSymbol( int which )
{
	// Determine which symbol we are talking about
	for ( idCollection::iterator i = m_SymbolMap.begin(); i != m_SymbolMap.end() && which > 0; i++ )
	{
		which --;
	}

	return m_pLibrary->m_Symbols.find(i->second);
}


void CLibraryDoc::DeleteSymbol(int which)
{
	CLibraryStore::symbolCollection::iterator it = getSymbol(which);

	// First we must delete the symbol from the library store...
	m_pLibrary->DeleteSymbol( it->second );


	// .. and adjust our cached information accordingly
	m_heights.clear();
}


void CLibraryDoc::EditSymbol(int which)
{
	// Is this a new symbol?
	if (which == -1)
	{
		CLibraryStoreNameSet new_symbol;
		new_symbol.Blank();
		new_symbol.lib = m_pLibrary;
		CTinyCadApp::EditSymbol( m_pLibrary, new_symbol );
	}
	else
	{
		// Determine which symbol we are talking about
		CLibraryStore::symbolCollection::iterator it = getSymbol(which);

		if (it != m_pLibrary->m_Symbols.end())
		{
			CTinyCadApp::EditSymbol( m_pLibrary, it->second );
		}
	}
}

void CLibraryDoc::DuplicateSymbol( int which )
{
	// Determine which symbol we are talking about
	CLibraryStore::symbolCollection::iterator it = getSymbol(which);

	if (it != m_pLibrary->m_Symbols.end())
	{
		// Load this symbol into a dummy document
		CTinyCadSymbolDoc tempDoc;
		tempDoc.setSymbol( m_pLibrary, it->second );

		CLibraryStoreNameSet *r = tempDoc.getSymbol();
		for (int i =0; i<r->GetNumRecords(); i++)
		{
			r->GetRecord(i).NameID = -1;
			if (i == 0)
			{
				r->GetRecord(i).name = "Copy of " + r->GetRecord(i).name;
			}
		}
		r->FilePos = -1;
		tempDoc.Store();
	}
}

void CLibraryDoc::CopySymbol(int which, CLibraryStore *target)
{
	// Determine which symbol we are talking about
	CLibraryStore::symbolCollection::iterator it = getSymbol(which);

	if (it != m_pLibrary->m_Symbols.end())
	{
		// Load this symbol into a dummy document
		CTinyCadSymbolDoc tempDoc;
		tempDoc.setSymbol( target, it->second );

		CLibraryStoreNameSet *r = tempDoc.getSymbol();
		for (int i =0; i<r->GetNumRecords(); i++)
		{
			r->GetRecord(i).NameID = -1;
		}
		r->FilePos = -1;
		tempDoc.Store();
	}

}

void CLibraryDoc::SymbolProperties( int which )
{
	// Determine which symbol we are talking about
	CLibraryStore::symbolCollection::iterator it = getSymbol(which);

	if (it != m_pLibrary->m_Symbols.end())
	{
		// Load this symbol into a dummy document
		CTinyCadSymbolDoc tempDoc;
		tempDoc.setSymbol( m_pLibrary, it->second );
		tempDoc.Store();
	}
}

void CLibraryDoc::OnFileExportlibrary() 
{
  // Get the file name for the parts list
  OPENFILENAME ofn;
  char szFile[256], szFileTitle[256];

  strcpy(szFile,m_pLibrary->m_name);
  strcat(szFile, ".xml");

  strcpy(szFileTitle,szFile);

  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner = AfxGetMainWnd()->m_hWnd;
  ofn.lpstrFilter = "XML Library file (*.xml)\0*.xml\0All files (*.*)\0*.*\0\0";
  ofn.lpstrCustomFilter = NULL;
  ofn.nMaxCustFilter = 0;
  ofn.nFilterIndex = 1;
  ofn.lpstrFile = szFile;
  ofn.nMaxFile = sizeof(szFile);
  ofn.lpstrFileTitle = szFileTitle;
  ofn.nMaxFileTitle = sizeof(szFileTitle);
  ofn.lpstrInitialDir = NULL;
  ofn.lpstrTitle = NULL;
  ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST  | OFN_NOCHANGEDIR;
  ofn.nFileOffset = 0;
  ofn.nFileExtension = 0;
  ofn.lpstrDefExt = "xml";
  ofn.lpstrTitle = "Export Library as XML";

  if (!GetSaveFileName(&ofn))
      	return;	

  m_pLibrary->SaveXML( szFile );
  
}

void CLibraryDoc::OnFileImportlibrary() 
{
  OPENFILENAME ofn;
  char szFile[256], szFileTitle[256];

  strcpy(szFile, "*.xml\0");

  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner = AfxGetMainWnd()->m_hWnd;
  ofn.lpstrFilter = "XML Library file (*.xml)\0*.xml\0All files (*.*)\0*.*\0\0";
  ofn.lpstrCustomFilter = NULL;
  ofn.nMaxCustFilter = 0;
  ofn.nFilterIndex = 1;
  ofn.lpstrFile = szFile;
  ofn.nMaxFile = sizeof(szFile);
  ofn.lpstrFileTitle = szFileTitle;
  ofn.nMaxFileTitle = sizeof(szFileTitle);
  ofn.lpstrInitialDir = NULL;
  ofn.lpstrTitle = NULL;
  ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
  ofn.nFileOffset = 0;
  ofn.nFileExtension = 0;
  ofn.lpstrDefExt = "xml";
  ofn.lpstrTitle = "Import";
 
  if (!GetOpenFileName(&ofn))
       	return;

  m_pLibrary->LoadXML( szFile );
}

void CLibraryDoc::OnContextExportsymbol() 
{

	// Determine which symbol we are talking about
	CLibraryStore::symbolCollection::iterator it = getSymbol( m_selected );
	if (it != m_pLibrary->m_Symbols.end())
	{

		CLibraryStoreNameSet &s = (*it).second;
		CSymbolRecord &r = s.GetRecord( 0 );

		// Get the file name for the symbol list
		OPENFILENAME ofn;
		char szFile[256], szFileTitle[256];

		strcpy(szFile,r.name);
		strcat(szFile, ".xml");

		strcpy(szFileTitle,szFile);

		ofn.lStructSize = sizeof(OPENFILENAME);
		ofn.hwndOwner = AfxGetMainWnd()->m_hWnd;
		ofn.lpstrFilter = "XML Library file (*.xml)\0*.xml\0All files (*.*)\0*.*\0\0";
		ofn.lpstrCustomFilter = NULL;
		ofn.nMaxCustFilter = 0;
		ofn.nFilterIndex = 1;
		ofn.lpstrFile = szFile;
		ofn.nMaxFile = sizeof(szFile);
		ofn.lpstrFileTitle = szFileTitle;
		ofn.nMaxFileTitle = sizeof(szFileTitle);
		ofn.lpstrInitialDir = NULL;
		ofn.lpstrTitle = NULL;
		ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST  | OFN_NOCHANGEDIR;
		ofn.nFileOffset = 0;
		ofn.nFileExtension = 0;
		ofn.lpstrDefExt = "xml";
		ofn.lpstrTitle = "Export Symbol as XML";

		if (!GetSaveFileName(&ofn))
			return;	

		m_pLibrary->SaveXML( szFile, (*it).first );	
	}
}

void CLibraryDoc::OnUpdateContextExportsymbol(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( getSelected() != -1 );	
}
