/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "tinycad.h"
#include "TinyCadView.h"
#include "diag.h"
#include "colour.h"
#include "option.h"
#include "LibraryView.h"
#include "LibraryDoc.h"
#include "revision.h"
#include "LibraryCollection.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLibraryView

IMPLEMENT_DYNCREATE(CLibraryView, CScrollView)

CLibraryView::CLibraryView()
{
}

CLibraryView::~CLibraryView()
{
}


BEGIN_MESSAGE_MAP(CLibraryView, CScrollView)
	//{{AFX_MSG_MAP(CLibraryView)
	ON_COMMAND(ID_SYMBOL_DELETESYMBOL, OnSymbolDeletesymbol)
	ON_COMMAND(ID_SYMBOL_EDITSYMBOL, OnSymbolEditsymbol)
	ON_UPDATE_COMMAND_UI(ID_SYMBOL_DELETESYMBOL, OnUpdateSymbolDeletesymbol)
	ON_UPDATE_COMMAND_UI(ID_SYMBOL_EDITSYMBOL, OnUpdateSymbolEditsymbol)
	ON_COMMAND(ID_SYMBOL_NEWSYMBOL, OnSymbolNewsymbol)
	ON_WM_LBUTTONDOWN()
	ON_WM_CREATE()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_RBUTTONDOWN()
	ON_COMMAND(ID_CONTEXT_DUPLICATESYMBOL, OnContextDuplicatesymbol)
	ON_COMMAND(ID_SYMBOL_SYMBOLPROPERTIES, OnSymbolSymbolproperties)
	ON_UPDATE_COMMAND_UI(ID_SYMBOL_SYMBOLPROPERTIES, OnUpdateSymbolSymbolproperties)
	ON_UPDATE_COMMAND_UI(ID_CONTEXT_DUPLICATESYMBOL, OnUpdateContextDuplicatesymbol)
	//}}AFX_MSG_MAP

	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)

	// Range of commands
	ON_COMMAND_RANGE( IDM_LIBCOPY, IDM_LIBCOPY+100, OnSendTo )

END_MESSAGE_MAP()


// The OnCreate function, called when the window is created
int CLibraryView::OnCreate( LPCREATESTRUCT q )
{
	int r = CView::OnCreate( q );

	if (r == 0)
	{
		  ClipboardFormat = RegisterClipboardFormat(CLIPBOARD_SY_FORMAT);
	}

	return r;
}

BOOL CLibraryView::IsClipboardAvailable()
{
	return ::IsClipboardFormatAvailable(ClipboardFormat);
}


/////////////////////////////////////////////////////////////////////////////
// CLibraryView drawing

void CLibraryView::OnDraw(CDC* pDC)
{
	CLibraryDoc * pDoc = GetDocument();

	CLibraryDoc::idCollection::iterator it = pDoc->m_SymbolMap.begin();
	CLibraryDoc::heightCollection::iterator h_it = pDoc->m_heights.begin();
	
	int current_y = 0;

	// The font for naming with
	CFont theFont;
	theFont.CreateFont(m_view_transform.intScale( 20 ),0,0,0,400,FALSE,FALSE,FALSE,
		ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS | CLIP_LH_ANGLES,
		DEFAULT_QUALITY,DEFAULT_PITCH | FF_MODERN,"Courier New");

	CFont theFont2;
	theFont2.CreateFont(m_view_transform.intScale( 16 ),0,0,0,400,FALSE,FALSE,FALSE,
		ANSI_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS | CLIP_LH_ANGLES,
		DEFAULT_QUALITY,DEFAULT_PITCH | FF_MODERN,"Courier New");


	CFont *oldFont = pDC->SelectObject( &theFont );

	TransformSnap s;
	s.SetGridSnap( FALSE );
	s.SetAccurateGrid(FineGrid);
	CTinyCadDoc doc;
	
	const int inter_spacing = 20;
	const int text_spacing = 25;
	int current_index = 0;
	current_y = 0;
	int start_y = 0;
	bool found_print_point = !pDC->IsPrinting();
	
	if (!pDC->IsPrinting())
	{
		start_y = GetScrollPosition().y;
	}


	int height = m_view_transform.intDeScale( pDC->GetDeviceCaps( VERTRES ) );
	int print_offset = 0;


	for (;it != pDoc->m_SymbolMap.end(); ++it )
	{
		Context q( pDC, m_view_transform );

		if (it == m_printing_it)
		{
			found_print_point = true;
			if (pDC->IsPrinting())
			{
				print_offset = current_y;
			}
		}

		
		if (h_it != pDoc->m_heights.end())
		{
			// Move on to the start point
			if (start_y > *h_it
				|| !found_print_point)
			{
				current_index ++;
				current_y = *h_it;
				++ h_it;
				continue;
			}
		}

		// Display the name of this symbol
		pDC->SelectObject( &theFont );
			
		// Now access the symbol and draw it next to the name
		CPoint p;
		CLibraryStoreNameSet &s = GetDocument()->getNameSet(it);
		CDesignFileSymbol *pSymbol = s.GetDesignSymbol( &doc, 0 );

		// Determine the rotation
		int orientation = s.orientation;


		drawingCollection method;
		if (pSymbol)
		{
			pSymbol->GetMethod( p, method );
		}
		else
		{
			p = CPoint(-75,-15);
		}

		if (orientation >= 2)
		{
			CPoint q = p;
			p.x = q.y;
			p.y = q.x;
		}

		int block_height = 0;
		if (h_it == pDoc->m_heights.end())
		{
			block_height = -p.y + inter_spacing*2;

			// Adjust for the amount of text we have...
			block_height = max( s.GetNumRecords() * 40, block_height );

			// Remember for later...
			pDoc->m_heights.insert( current_y + block_height );
		}
		else
		{
			block_height = *h_it - current_y;
		}



	

		// Fill in the background rectangle
		q.theTransform.SetOrigin( CPoint( 0, print_offset-current_y) );
		if (!pDC->IsPrinting())
		{
			if (current_index == pDoc->getSelected())
			{
				q.SelectBrush( cOFFPAGE );
			}
			else
			{
				q.SelectBrush( cWHITE );
			}
			q.SelectPen( cBLACK,0,0);
			q.Rectangle( CRect(0,0,1024, block_height ));
		}

		// Draw the name
		q.SetTextAlign(TA_LEFT | TA_BOTTOM | TA_NOUPDATECP);
		q.SetBkMode( TRANSPARENT );
		q.SetTextColor( RGB(0,0,0) );

		// Do this for each of the names in the symbol set
		int text_pos_y = 20;
		for (int i =0; i < s.GetNumRecords(); i++)
		{
			CSymbolRecord &r = s.GetRecord( i );

			pDC->SelectObject( &theFont );
			q.TextOut( 5, text_pos_y, r.name );

			pDC->SelectObject( &theFont2 );
			q.TextOut( 5, text_pos_y + 16, r.description );
			text_pos_y += 40;
		}


		// Now display the symbol
		q.theTransform.SetOrigin( CPoint( -500, print_offset-current_y) );
		CPoint old;
		switch (orientation)
		{
		case 3:
			old = q.theTransform.SetTRM( CPoint(p.x,-p.y+inter_spacing), CPoint(0,0), orientation );
			break;
		default:
			old = q.theTransform.SetTRM( CPoint(0,-p.y+inter_spacing), CPoint(0,0), orientation );
			break;
		}

		 

		if (pSymbol)
		{
			drawingIterator paint_it = method.begin();
			while (paint_it != method.end())
			{
				(*paint_it)->Paint( q, draw_normal );
				++ paint_it;
			}

			delete pSymbol;
		}

		q.theTransform.EndTRM( old );


		current_index ++;
		current_y += block_height;
		if (h_it != pDoc->m_heights.end())
		{
			++ h_it;
		}


		if (h_it != pDoc->m_heights.end())
		{
			if (pDC->IsPrinting())
			{
				if (current_y - start_y - print_offset + block_height > height)
				{
					break;
				}
			}
			else
			{
				if (current_y - start_y - print_offset > height)
				{
					break;
				}
			}
		}

	}

	pDC->SelectObject( oldFont );


	if (pDC->IsPrinting())
	{
		m_printing_it = it;
	}

	SetScrollSizes( MM_TEXT, GetDocument()->GetDocSize() );
}

/////////////////////////////////////////////////////////////////////////////
// CLibraryView diagnostics

#ifdef _DEBUG
void CLibraryView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CLibraryView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CLibraryDoc* CLibraryView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CLibraryDoc)));
	return (CLibraryDoc*)m_pDocument;
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CLibraryView message handlers

void CLibraryView::OnInitialUpdate() 
{
	CScrollView::OnInitialUpdate();
	
	SetScrollSizes( MM_TEXT, GetDocument()->GetDocSize() );
}

void CLibraryView::OnSymbolNewsymbol() 
{
	GetDocument()->EditSymbol( -1 );		
}

void CLibraryView::OnSymbolDeletesymbol() 
{
	if (AfxMessageBox( IDS_CONFIRM_DELETE, MB_YESNO ) == IDYES)
	{
		GetDocument()->DeleteSymbol( GetDocument()->getSelected() );
		Invalidate();
	}
}

void CLibraryView::OnSymbolEditsymbol() 
{
	GetDocument()->EditSymbol( GetDocument()->getSelected() );
	
}


void CLibraryView::OnUpdateSymbolDeletesymbol(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( GetDocument()->getSelected() != -1 );
}

void CLibraryView::OnUpdateSymbolEditsymbol(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( GetDocument()->getSelected() != -1 );
	
}




/////////////////////////////////////////////////////////////////////////////
// CLibraryView printing

BOOL CLibraryView::OnPreparePrinting(CPrintInfo* pInfo) 
{

	// default preparation
	return DoPreparePrinting(pInfo);
}

void CLibraryView::OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo) 
{
	// Count the symbols and determine the page count and splits
	CLibraryDoc * pDoc = GetDocument();
	CLibraryDoc::idCollection::iterator it = pDoc->m_SymbolMap.begin();


	TransformSnap s;
	s.SetGridSnap( FALSE );
	CTinyCadDoc doc;
	m_view_transform.SetZoom( 500 );

	const int inter_spacing = 20;
	const int text_spacing = 25;
	int current_y = 0;

	int height = m_view_transform.intDeScale( pDC->GetDeviceCaps( VERTRES ) );
	int pages = 1;

	CLibraryDoc::heightCollection::iterator h_it = pDoc->m_heights.begin();
	while (it != pDoc->m_SymbolMap.end())
	{

		// Is it time to move onto the next page?
		if ( *h_it - current_y > height)
		{
			pages ++;
			current_y += height;
		}

		// Move on to the next symbol
		it ++;
		h_it ++;
	}

	pInfo->SetMinPage( 1 );
	pInfo->SetMaxPage( pages );

	m_printing_it = pDoc->m_SymbolMap.begin();
	
	CScrollView::OnBeginPrinting(pDC, pInfo);
}

void CLibraryView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
	if (pInfo)
	{

		// Count the symbols and determine the page count and splits
		CLibraryDoc * pDoc = GetDocument();
		m_view_transform.SetZoom( 500 );
		TransformSnap s;
		s.SetGridSnap( FALSE );
		CTinyCadDoc doc;

		const int inter_spacing = 20;
		const int text_spacing = 25;
		int current_y = 0;

		int height = m_view_transform.intDeScale( pDC->GetDeviceCaps( VERTRES ) );
		int pages = 1;

		m_printing_it = pDoc->m_SymbolMap.begin();
		CLibraryDoc::heightCollection::iterator h_it = pDoc->m_heights.begin();

		if (pInfo->m_nCurPage > 1)
		{
			while (m_printing_it != pDoc->m_SymbolMap.end())
			{
				// Is it time to move onto the next page?
				if ( *h_it - current_y > height)
				{
					pages ++;
					current_y += height;

					if (pages == pInfo->m_nCurPage)
					{
						break;
					}
				}

				// Move on to the next symbol
				m_printing_it ++;
				h_it ++;
			}
		}
	}
	else
	{
		m_view_transform.SetZoom( 100 );
	}


	CScrollView::OnPrepareDC(pDC, pInfo);
}

void CLibraryView::OnEndPrinting(CDC* pDC, CPrintInfo* pInfo) 
{
	// TODO: Add your specialized code here and/or call the base class	
	CScrollView::OnEndPrinting(pDC, pInfo);
}



/////////////////////////////////////////////////////////////////////////////
// The user is selecting a new object

void CLibraryView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// Determine the current scroll limits
	int y_hit = GetScrollPosition().y + point.y;

	// Now determine which object this lie within
	int current_index = 0;
	std::set<int>::iterator it = GetDocument()->m_heights.begin();
	int last_y_height;
	while (it != GetDocument()->m_heights.end())
	{
		// Is this a hit?
		if (*it > y_hit)
			break;

		// Move on
		current_index ++;
		last_y_height = *it;
		it ++;
	}

	GetDocument()->setSelected( current_index );
	Invalidate();
	
	CScrollView::OnLButtonDown(nFlags, point);
}


void CLibraryView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	OnSymbolEditsymbol();
	
	CScrollView::OnLButtonDblClk(nFlags, point);
}


void CLibraryView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	// First select the clicked on symbol...
	OnLButtonDown( nFlags, point );

	ClientToScreen( &point );

	// Now bring up the context menu
	CMenu menu;
	menu.LoadMenu( IDR_LIBVIEW );

	CMenu lib_menu;
	lib_menu.CreatePopupMenu();

	CLibraryCollection::FillLibraryNames( IDM_LIBCOPY, &lib_menu );
	menu.ModifyMenu( ID_CONTEXT_COPY_TO, MF_POPUP, (unsigned int)lib_menu.m_hMenu, _T("Copy to") );
	lib_menu.Detach();
	menu.GetSubMenu(0)->TrackPopupMenu( TPM_LEFTALIGN|TPM_RIGHTBUTTON,
		point.x,point.y, AfxGetMainWnd(), NULL );
	
	CScrollView::OnRButtonDown(nFlags, point);
}

void CLibraryView::OnSendTo(UINT nCmd)
{
	// Did we find the library?
	CLibraryStore *lib = CLibraryCollection::GetLibraryByIndex( nCmd - IDM_LIBCOPY );

	if (lib)
	{
		// First can this library be upgraded?
		if (lib->MustUpgrade())
		{
			if (AfxMessageBox(IDS_MUSTUPGRADE, MB_YESNO ) != IDYES)
			{
				return;
			}

			lib = CLibraryCollection::Upgrade(lib);
			if (!lib)
			{
				return;
			}
		}


		// Now copy the symbol to the specified library...
		GetDocument()->CopySymbol( GetDocument()->getSelected(), lib );
	}
}


void CLibraryView::OnContextDuplicatesymbol() 
{
	// Now copy the symbol to the specified library...
	GetDocument()->DuplicateSymbol( GetDocument()->getSelected() );
}

void CLibraryView::OnUpdateContextDuplicatesymbol(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( GetDocument()->getSelected() != -1 );
}


void CLibraryView::OnSymbolSymbolproperties() 
{
	// Symbol properties
	GetDocument()->SymbolProperties( GetDocument()->getSelected() );

}

void CLibraryView::OnUpdateSymbolSymbolproperties(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( GetDocument()->getSelected() != -1 );	
}



