/*
 * Project:		TinyCAD program for schematic capture
 *				http://tinycad.sourceforge.net
 * Copyright:	 1994-2005 Matt Pyne
 * License:		Lesser GNU Public License 2.1 (LGPL)
 *				http://www.opensource.org/licenses/lgpl-license.html
 */

#include "stdafx.h"
#include "ListOfMetaFiles.h"

//=========================================================================
//== ctor/dtor/initializing                                              ==
//=========================================================================

//-------------------------------------------------------------------------
ListOfMetaFiles::ListOfMetaFiles()
{
	Init( NULL );
}
//-------------------------------------------------------------------------
ListOfMetaFiles::ListOfMetaFiles( const ListOfMetaFiles& o )
{
	Init( o.m_oMetaFile );
}
//-------------------------------------------------------------------------
ListOfMetaFiles::ListOfMetaFiles( HENHMETAFILE s )
{
	Init( s );
}
//-------------------------------------------------------------------------
ListOfMetaFiles::~ListOfMetaFiles()
{
}
//-------------------------------------------------------------------------
//-- The default line style
void ListOfMetaFiles::Init( HENHMETAFILE s )
{
	m_oMetaFile = s;
}
//-------------------------------------------------------------------------

//=========================================================================
//== accessor/mutator                                                    ==
//=========================================================================

//-------------------------------------------------------------------------
//-- Get the data associated with this resource
HENHMETAFILE ListOfMetaFiles::GetMetaFile() const
{
	return m_oMetaFile;
};
//-------------------------------------------------------------------------

//=========================================================================
//== serialization                                                       ==
//=========================================================================

//-------------------------------------------------------------------------
void ListOfMetaFiles::ReadNative( CStream& oStream )
{
	UINT32 	nSize		= -1;
  	BYTE*	naBuffer	= NULL;

	Init( NULL );

	oStream >> nSize;
	naBuffer = new BYTE[ nSize ];
  	oStream.Read( naBuffer, nSize );

  	// Create the new metafile
  	m_oMetaFile = SetEnhMetaFileBits( nSize, naBuffer );

	delete[] naBuffer;
}
//-------------------------------------------------------------------------
void ListOfMetaFiles::Read( CXMLReader& xml, hMETAFILE& nID )
{
	CString name;
	UINT 	nSize 		= 0;
	BYTE*	naBuffer 	= NULL;

	Init( NULL );

	xml.getAttribute( "id", nID );
	xml.intoTag();

	while (xml.nextTag( name ))
	{
		if (name == "UUENCODE")
		{
			xml.getChildDataUUdecode( naBuffer, nSize );
		}
	}

	xml.outofTag();

	// Now create the new metafile
	m_oMetaFile = SetEnhMetaFileBits( nSize, naBuffer );

	delete[] naBuffer;
}
//-------------------------------------------------------------------------
void ListOfMetaFiles::Write( CXMLWriter& xml ) const
{
  // Put the meta-file into a memory buffer
  UINT 	nSize 		= GetEnhMetaFileBits( m_oMetaFile, 0, NULL );
  BYTE*	naBuffer	= new BYTE[ nSize ];

  GetEnhMetaFileBits( m_oMetaFile, nSize, naBuffer );

  // Write the size to the archive
  xml.addChildDataUUencode( naBuffer, nSize );

  // Now discard the buffer
  delete[] naBuffer;
}
//--------------------------------------------------------------------------
void ListOfMetaFiles::SaveItemXML( CTinyCadDoc*, CXMLWriter& xml )
{
	Write( xml );
}
//--------------------------------------------------------------------------
void ListOfMetaFiles::LoadItemXML( CTinyCadDoc*, CXMLReader& xml )
{
	ListOfMetaFiles	oMeta;
	hRESOURCE		nID;

	oMeta.Read( xml, nID );

	Add( new ListOfMetaFiles(oMeta), nID );
}
//--------------------------------------------------------------------------
void ListOfMetaFiles::LoadItem(CTinyCadDoc*, CStream& oStream, hRESOURCE n )
{
	ListOfMetaFiles oMeta;

	oMeta.ReadNative( oStream );

	Add( new ListOfMetaFiles(oMeta), n );
}
//--------------------------------------------------------------------------

//=========================================================================
//== Comparison                                                          ==
//=========================================================================

//-------------------------------------------------------------------------
BOOL ListOfMetaFiles::Compare( CDocResource* o )
{
	#define  pMetaFile (((ListOfMetaFiles *)o)->m_oMetaFile)

	return m_oMetaFile == pMetaFile;
}
//-------------------------------------------------------------------------
