/*
 * Project:		TinyCAD program for schematic capture
 *				http://tinycad.sourceforge.net
 * Copyright:	 1994-2005 Matt Pyne
 * License:		Lesser GNU Public License 2.1 (LGPL)
 *				http://www.opensource.org/licenses/lgpl-license.html
 */

#include "stdafx.h"
#include "LibraryCollection.h"
#include "ListOfSymbols.h"

//=========================================================================
//== ctor/dtor/initializing                                              ==
//=========================================================================

//-------------------------------------------------------------------------
ListOfSymbols::ListOfSymbols()
: super()
{
	Init();
}
//-------------------------------------------------------------------------
ListOfSymbols::ListOfSymbols( const ListOfSymbols& o )
{
	Init( o.m_pSymbol );
}
//-------------------------------------------------------------------------
ListOfSymbols::ListOfSymbols( CDesignFileSymbol* pSymbol )
: super()
{
	Init( pSymbol );
}
//-------------------------------------------------------------------------
ListOfSymbols::~ListOfSymbols()
{
	delete m_pSymbol;
}
//-------------------------------------------------------------------------
void ListOfSymbols::Init()
{
	Init( NULL );
}
//-------------------------------------------------------------------------
void ListOfSymbols::Init( CDesignFileSymbol* pSymbol )
{
	m_pSymbol = pSymbol;
}
//-------------------------------------------------------------------------

//=========================================================================
//== accessor/mutator                                                    ==
//=========================================================================

//-------------------------------------------------------------------------
CDesignFileSymbol* ListOfSymbols::GetSymbol()
{
	return m_pSymbol;
}
//-------------------------------------------------------------------------
// Reload from the library
void ListOfSymbols::ResetSymbol()
{
	// Extract this symbol from the library and see what happens...
	CLibraryStoreNameSet* new_symbol = CLibraryCollection::GetSymbol( m_pSymbol->name );

	if( new_symbol )
	{
		CTinyCadDoc *pDesign = m_pSymbol->GetDesign();
		delete m_pSymbol;
		m_pSymbol = new_symbol->GetDesignSymbol( pDesign, 0 );
	}
}
//-------------------------------------------------------------------------

//=========================================================================
//== serialization                                                       ==
//=========================================================================

////-------------------------------------------------------------------------
////-- Save the symbol with the document
//void ListOfSymbols::ReadNative( CStream& oStream, CTinyCadDoc* pDesign  )
//{
//	//drawingCollection method;
//
//	Init();
//
//	//m_pSymbol = new CDesignFileSymbol( pDesign, method, CPoint(0,0) );
//	m_pSymbol->Load( pDesign, oStream );
//}
////-------------------------------------------------------------------------
//void ListOfSymbols::Read( CXMLReader& xml, hRESOURCE& nID, CTinyCadDoc* pDesign  )
//{
//	//drawingCollection colMethod;
//
//	Init();
//
//	xml.getAttribute( "id", nID );
//	xml.intoTag();
//	//m_pSymbol = new CDesignFileSymbol( pDesign, colMethod, CPoint(0,0) );
//	m_pSymbol->LoadXML( pDesign, xml );
//	xml.outofTag();
//}
////-------------------------------------------------------------------------
//void ListOfSymbols::Write( CXMLWriter& xml ) const
//{
//	m_pSymbol->SaveXML( xml );
//}
////-------------------------------------------------------------------------
//void ListOfSymbols::SaveItemXML( CTinyCadDoc*, CXMLWriter &xml)
//{
//	Write( xml );
//}
////-------------------------------------------------------------------------
//void ListOfSymbols::LoadItemXML( CTinyCadDoc* pDesign, CXMLReader& xml )
//{
//	hRESOURCE 			nID;
//	ListOfSymbols		oSymbol;
//
//	oSymbol.Read( xml, nID,  pDesign );
//
//	Add( new ListOfSymbols(oSymbol), nID );
//}
////-------------------------------------------------------------------------
//void ListOfSymbols::LoadItem( CTinyCadDoc* pDesign, CStream& oStream, hRESOURCE n)
//{
//	ListOfSymbols oSymbol;
//
//	oSymbol.ReadNative( oStream, pDesign );
//
//	Add( new ListOfSymbols(oSymbol), n );
//}
//-------------------------------------------------------------------------
void ListOfSymbols::SaveItemXML( CTinyCadDoc *pDesign, CXMLWriter &xml)
{
	// Save the symbol with the document...
	m_pSymbol->SaveXML( xml );
}
//-------------------------------------------------------------------------
void ListOfSymbols::LoadItemXML(CTinyCadDoc *pDesign, CXMLReader &xml)
{
	hRESOURCE n;
	CString name;
	drawingCollection method;
	CDesignFileSymbol *psymbol = new CDesignFileSymbol( pDesign, method, CPoint(0,0) );

	xml.getAttribute( "id", n );
	xml.intoTag();

	psymbol->LoadXML( pDesign, xml );

	xml.outofTag();

	ListOfSymbols *nFS = new ListOfSymbols(psymbol);
	Add(nFS,n);
}
//-------------------------------------------------------------------------
void ListOfSymbols::LoadItem(CTinyCadDoc *pDesign, CStream &theFile, hRESOURCE n)
{
	// Save the symbol with the document...
	drawingCollection method;
	CDesignFileSymbol *psymbol = new CDesignFileSymbol( pDesign, method, CPoint(0,0) );
	psymbol->Load( pDesign, theFile );

	ListOfSymbols *nFS = new ListOfSymbols(psymbol);
	Add(nFS,n);
}
//-------------------------------------------------------------------------

//=========================================================================
//== Comparison                                                          ==
//=========================================================================

//-------------------------------------------------------------------------
// The resource for line styles
BOOL ListOfSymbols::Compare(CDocResource *o)
{
	CDesignFileSymbol *pSymbol = static_cast<CDesignFileSymbol*>(((ListOfSymbols *)o)->m_pSymbol);
	return m_pSymbol->name == pSymbol->name && m_pSymbol->FilePos == pSymbol->FilePos;
}
//-------------------------------------------------------------------------
