/*
 * Project:		TinyCAD program for schematic capture
 *				http://tinycad.sourceforge.net
 * Copyright:	 1994-2004 Matt Pyne
 * License:		Lesser GNU Public License 2.1 (LGPL)
 *				http://www.opensource.org/licenses/lgpl-license.html
 */

#include "stdafx.h"
#include "TinyCadView.h"
#include "TinyCad.h"
#include "MainFrm.h"
#include "StreamClipboard.h"
#include "DlgAbout.h"
#include "DetailsPropertySheet.h"

// The menu commands!
// NOTE: many menu commands are in-line in main.h


//////// the FILE menu ////////


void CTinyCadView::OnFileDesign()
{
	CDetailsPropertySheet theDialog( GetDocument(), this );
	
  if (theDialog.DoModal() == IDOK)
  {
	GetCurrentDocument()->SetModifiedFlag( TRUE );
	GetCurrentDocument()->Invalidate();
  }
}


void CTinyCadView::OnFileImport() 
{ 
	GetCurrentDocument()->SelectObject(new CDrawEditItem(GetCurrentDocument())); 
	GetCurrentDocument()->UnSelect(); 
	CDrawBlockImport *pImport = new CDrawBlockImport(GetCurrentDocument());
	GetCurrentDocument()->SelectObject(pImport); 
	pImport->Import( TRUE );
}


void CTinyCadView::OnFilePageSet()
{
  // Get rid of any drawing tool
  GetCurrentDocument()->SelectObject(new CDrawEditItem(GetCurrentDocument()));

  CDlgPageSizeBox theDialog(this,GetCurrentDocument()->GetDetails().GetPageBoundsAsPoint());
  if (theDialog.DoModal()==IDOK) {
	GetCurrentDocument()->GetDetails().SetPageBounds(theDialog.GetSize());
	Invalidate();
  }
}




////// The Find Menu //////

extern CDlgERCListBox theERCListBox;


// Find a string in this design
void CTinyCadView::OnFindFind()
{
	// Get rid of any drawing tool at this moment
	GetCurrentDocument()->SelectObject(new CDrawEditItem(GetCurrentDocument()));

	// Do the dialog
	CDlgFindBox theDialog(this);

	if (theDialog.DoModal()!=IDOK)
		return;

	// Get the string to be found
	CString FindString = theDialog.GetString();

	// Now convert it to lower case only (as this will be a case insensive search)
	FindString.MakeLower();

	
	// Delete all the errors which are currently in the design
	theERCListBox.Close();
	GetCurrentDocument()->DeleteErrors();
	theERCListBox.Open(GetCurrentDocument());
	int CurrentError = 0;

	// Now look for this string
	CString theFoundString = "";

	drawingIterator it = GetCurrentDocument()->GetDrawingBegin();
	while (it != GetCurrentDocument()->GetDrawingEnd()) 
	{
		CDrawingObject *pointer = *it;

		theFoundString = pointer->Find(FindString);
		if (theFoundString!="") {
			char buffer[STRLEN];
			strcpy(buffer,pointer->GetName());
			strcat(buffer,": ");
			strcat(buffer,theFoundString);
			GetCurrentDocument()->Add(new CDrawError(GetCurrentDocument(),pointer->a,CurrentError++));
			theERCListBox.AddString(buffer);
		}
		
		++ it;
	}

	// Was the string found?
	if (CurrentError == 0) {
		Message(IDS_NOFIND);
		theERCListBox.Close();
	}
}


// Set a tag in the design
void CTinyCadView::OnFindSetTag()
{
	CDlgTagBox theDialog(this);

	if (theDialog.DoModal()!=IDOK)
		return;

	// Get the name of the new tag
	CString NewTagName = theDialog.GetString();

	// Now create a new tag with this name
	CDrawTag *newTag = new CDrawTag(GetCurrentDocument(),theTransform.GetOrigin(),NewTagName);

	GetCurrentDocument()->Add(newTag);
}



// Jump to or delete a tag in this design
void CTinyCadView::OnFindJumpTag()
{			   
 	CDlgJumpTagBox theDialog(this,GetCurrentDocument());

	if (theDialog.DoModal()!=IDOK)
		return;

	// Now jump to that tag
	drawingIterator it = GetCurrentDocument()->GetDrawingBegin();
	while (it != GetCurrentDocument()->GetDrawingEnd()) 
	{
		CDrawingObject *pointer = *it;
		if (pointer->GetType()==xTag && ((CDrawTag *)pointer)->GetTagName()==theDialog.GetString())
		{
			theTransform.SetOrigin(pointer->a);
			Invalidate();
		}

		++ it;
	}
}




//////// The VIEW menu ////////

void CTinyCadView::OnViewZoomIn()
{
  CMenu *pMenu = GetMenu();
  LONG NewZoom=theTransform.Scale(1000)*2;

  if (NewZoom>5000)
	NewZoom=5000;
  NewZoom = NewZoom /10;

  SetZoom(NewZoom);

  // Centre at the mouse position
  SetScrollCentre(MousePosition);
}

void CTinyCadView::OnViewZoomOut()
{
  LONG NewZoom;

  NewZoom=theTransform.Scale(1000)/2;
  if (NewZoom<250)
	NewZoom=250;
  SetZoom(NewZoom/10);
}







//////// The TOOLS menu /////////



// Get an object
void CTinyCadView::OnSelectGet()
{
  CLibraryStoreSymbol *theSymbol = static_cast<CMainFrame*>(AfxGetMainWnd())->GetSelectSymbol();
  if (theSymbol==NULL) 
  {
	return;
  }  

  GetCurrentDocument()->SelectSymbol( theSymbol );
}








//////// The Help Menu ////////

void CTinyCadView::OnAbout()
{
	CDlgAbout().DoModal();
}





////// The EDIT Menu //////



 void CTinyCadView::OnEditCut()
{
	if (GetCurrentDocument()->GetEdit()->GetType() != xEditItem)
		return;

	((CDrawEditItem *)GetCurrentDocument()->GetEdit())->ReleaseSelection();
	OnEditCopy();
	GetCurrentDocument()->BeginNewChangeSet();
	GetCurrentDocument()->SelectDelete();
	Invalidate();
}


// Paste...
void CTinyCadView::OnEditPaste()
{
	OpenClipboard();
	if (IsClipboardAvailable())
	{
		GetCurrentDocument()->BeginNewChangeSet();
		GetCurrentDocument()->SelectObject(new CDrawEditItem(GetCurrentDocument()));

		// Now open the file
		CStreamClipboard stream;

		if (stream.ReadFromClipboard( ClipboardFormat ))
		{
			GetCurrentDocument()->SelectObject( NULL );

			if (GetCurrentDocument()->Import(stream))
			{
				GetCurrentDocument()->PostPaste();
				CDrawBlockImport *pImport = new CDrawBlockImport(GetCurrentDocument());
				GetCurrentDocument()->SelectObject(pImport); 
				pImport->Import( FALSE );
			}
			else
			{
				GetCurrentDocument()->SelectObject( new CDrawEditItem(GetCurrentDocument()) );	
			}
		}

	}
	else if (::IsClipboardFormatAvailable( CF_ENHMETAFILE ))
	{
		// It is a meta-file, we paste these as a new type of object
		HENHMETAFILE clipboard_data = (HENHMETAFILE)::GetClipboardData( CF_ENHMETAFILE );
		CDrawMetaFile *pObject = new CDrawMetaFile( GetCurrentDocument() );
		pObject->setMetaFile( clipboard_data );
		GetCurrentDocument()->Add( pObject );
		GetCurrentDocument()->UnSelect();
		CDrawBlockImport *pImport = new CDrawBlockImport(GetCurrentDocument());
		GetCurrentDocument()->SelectObject(pImport); 
		GetCurrentDocument()->Select( pObject );
		pImport->Import( FALSE );
	}
	else
	{
		AfxMessageBox("Cannot paste");
	}

	CloseClipboard();
}

 void CTinyCadView::OnDestroyClipboard()
{
	if (IsClipboardAvailable()) 
	{
		char theName[MAX_PATH],TempPath[MAX_PATH];
		GetTempPath(sizeof(TempPath),TempPath);
		GetTempFileName(TempPath,"$DS",42,theName);

		::DeleteFile(theName);
	}

	CWnd::OnDestroyClipboard();	
}

BOOL CTinyCadView::IsClipboardAvailable()
{
	return ::IsClipboardFormatAvailable(ClipboardFormat);
}

void CTinyCadView::OnEditDuplicate()
{
	if (GetCurrentDocument()->GetEdit()->GetType() != xEditItem)
		return;
	OnEditCopy();
	OnEditPaste();
}

void CTinyCadView::OnEditCopy()
{
	if (GetCurrentDocument()->GetEdit()->GetType() != xEditItem)
		return;

	OpenClipboard();
	EmptyClipboard();

	CClientDC ref_dc(this);
	HENHMETAFILE hmeta = GetCurrentDocument()->CreateMetafile( ref_dc );

	if (hmeta)
	{
		SetClipboardData(CF_ENHMETAFILE,hmeta);
	}

	// Save the selected area
	CStreamClipboard stream;
	CXMLWriter xml( &stream );
	if (GetCurrentDocument()->IsSelected())
	{
		GetCurrentDocument()->SaveXML(xml,TRUE,TRUE);
	}
	else
	{
		GetCurrentDocument()->SaveXML(xml,TRUE,FALSE);
	}
	stream.SaveToClipboard(ClipboardFormat);

	CloseClipboard();
}


void CTinyCadView::OnEditCopyto() 
{
	if (GetCurrentDocument()->IsSelected())
	{
		GetCurrentDocument()->Save(TRUE,TRUE);
	}
}



void CTinyCadView::OnSelectArc() 
{ 
	g_EditToolBar.m_LineEdit.SetArcAngle();
	GetCurrentDocument()->SelectObject(new CDrawPolygon(GetCurrentDocument())); 
}

void CTinyCadView::OnSelectPolygon() 
{ 
	g_EditToolBar.m_LineEdit.SetLineAngle();
	GetCurrentDocument()->SelectObject(new CDrawPolygon(GetCurrentDocument())); 
}


void CTinyCadView::OnSelectJunction()
{ 
	if (!GetCurrentDocument()->GetOptions()->GetAutoJunc())
	{
		GetCurrentDocument()->SelectObject(new CDrawJunction(GetCurrentDocument())); 
	}
	else
	{
		AfxMessageBox( IDS_NOJUNCTION );
	}
}

