/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



// This handles the update of the main window
#include "stdafx.h"
#include "TinyCad.h"
#include "TinyCadView.h"
#include "colour.h"
#include "revision.h"
#include "option.h"
#include "ruler.h"


/////////////////////////////////////////////////////////////////////////////
// CTinyCadView drawing

void CTinyCadView::OnDraw(CDC* pDC)
{
	CTinyCadDoc* pDoc = GetCurrentDocument();
	CDC BitmapDC;
	CBitmap *old_bitmap;

  int selected;

  CRect client;
  if (pDC->IsKindOf(RUNTIME_CLASS(CPaintDC)))
  {
	  client  = static_cast<CPaintDC*>(pDC)->m_ps.rcPaint;
  }
  else
  {
	 GetClientRect( &client );
  }

  // Are we going to use off-screen drawing?
  BOOL osb = 
		!pDC->IsPrinting()
	&&  m_use_offscreen_drawing
	&&  CreateBitmap( *pDC, client.Width(), client.Height() );

  if (osb)
  {
	BitmapDC.CreateCompatibleDC( pDC );
	old_bitmap = BitmapDC.SelectObject( &m_bitmap );
  }

  Context dc(osb ? &BitmapDC : pDC,theTransform,this);

  CPoint origin = theTransform.GetOrigin();

  if (osb)
  {
	dc.theTransform.SetPixelOffset( CPoint( -client.left, -client.top ) );
  }


  if (pDC->IsPrinting())
  {
	  dc.SetBlack(((CTinyCadApp*)AfxGetApp())->GetPrintBandW());
  }



  CPoint Start,End;
  CRect rect;
  GetClientRect( &rect );
  TransformSnap snap;
  snap.SetGridSnap( FALSE );
  Start=theTransform.DeScale(snap,CPoint(rect.left,rect.top));
  End=theTransform.DeScale(snap,CPoint(rect.right,rect.bottom));


  // Is any of this region in the off-page area?
  if (!pDC->IsPrinting())
  {

	  // Paint the region white
	  dc.SelectBrush(cWHITE);
	  dc.SelectPen(PS_SOLID,1,cWHITE);
	  dc.Rectangle(CRect(Start.x - 2,Start.y - 2,End.x + 2,End.y + 2));

	  dc.SelectBrush(cOFFPAGE);
	  dc.SelectPen(PS_SOLID,1,cOFFPAGE);

	  if (End.x>GetCurrentDocument()->GetDetails().GetPageBoundsAsPoint().x) {
		CPoint a=CPoint(GetCurrentDocument()->GetDetails().GetPageBoundsAsPoint().x,0);
		dc.Rectangle(CRect(a.x,a.y,End.x,End.y));
	  }
	  if (End.y>GetCurrentDocument()->GetDetails().GetPageBoundsAsPoint().y) {
		CPoint a=CPoint(Start.x,GetCurrentDocument()->GetDetails().GetPageBoundsAsPoint().y);
		dc.Rectangle(CRect(a.x,a.y,End.x,End.y));
	  }
	  if (Start.x<0)
		dc.Rectangle(CRect(0,Start.y,Start.x,End.y));
	  if (Start.y<0)
		dc.Rectangle(CRect(Start.x,0,End.x,Start.y));

	  // Fill this region with a grid
	  int grid = GetCurrentDocument()->m_snap.GetGrid();
	  int SGrid = dc.theTransform.Scale(grid);
	  if (GetCurrentDocument()->GetOptions()->ShowGrid() && SGrid>5) {
		int xi,yi;
		int x = dc.theTransform.GetOrigin().x;
		int y = dc.theTransform.GetOrigin().y;

		BOOL doall = SGrid>10, showX = (GetCurrentDocument()->m_snap.GetAccurateGrid()==FineGrid);

		TransformSnap s = GetCurrentDocument()->m_snap;
		s.SetGridSnap(TRUE);

   		for (int xp=x>=0 ? x : 0;xp<End.x && xp<GetCurrentDocument()->GetDetails().GetPageBoundsAsPoint().x;xp+=grid) {
			for (int yp=y>=0 ? y : 0;yp<End.y && yp<GetCurrentDocument()->GetDetails().GetPageBoundsAsPoint().y;yp+=grid) {
				xi = s.Snap(xp);
				yi = s.Snap(yp);
				if ((xi%(NormalGrid / 10)) == 0 && (yi%(NormalGrid/10)) == 0) {
					dc.SetPixel(CPoint(xi,yi),0);
					if (showX) {
						dc.SetPixel(CPoint(xi+1,yi),0);
						dc.SetPixel(CPoint(xi-1,yi),0);
						dc.SetPixel(CPoint(xi,yi+1),0);
						dc.SetPixel(CPoint(xi,yi-1),0);
					}
				} else if (doall)
					dc.SetPixel(CPoint(xi,yi),0);
			}
		}
	  }
  }



  Start -= CPoint(10,10);
  End += CPoint(10,10);


	GetCurrentDocument()->GetSelectBegin();

	drawingIterator it = GetCurrentDocument()->GetDrawingBegin();
	while (it != GetCurrentDocument()->GetDrawingEnd()) 
	{
		CDrawingObject *obj = *it;

		selected=!pDC->IsPrinting() && GetCurrentDocument()->IsSelected( obj );
		paint_options options = selected ? draw_selected : draw_normal;

		if (!pDC->IsPrinting() || !obj->IsConstruction())
		{
			if (pDC->IsPrinting() || obj->IsInside(Start.x, End.x, Start.y, End.y ))
			{
				obj->Paint(dc,options);
			}
		}

		++ it;
	}
 

  // Now draw the selectable object, so it stands out...
  CDrawingObject *obj = GetCurrentDocument()->GetSelectable();
  if ( obj != NULL && !GetCurrentDocument()->IsSelected( obj ))
  {
	  paint_options options = draw_selectable;
	  GetCurrentDocument()->GetSelectable()->Paint( dc, options );
  }

  // If only one item is selected then just draw its handles now
  if (GetCurrentDocument()->IsSingleItemSelected())
  {
	  GetCurrentDocument()->GetSingleSelectedItem()->PaintHandles( dc );
  }

  // if necessary turn back on the current object to be edited
  if (GetCurrentDocument()->GetEdit()!= NULL)
	GetCurrentDocument()->GetEdit()->Paint(dc,draw_selected);

  // Draw the design details
  GetCurrentDocument()->Display( dc );


  if (osb)
  {
	pDC->BitBlt( client.left, client.top, client.Width(), client.Height(),
		&BitmapDC, 0,0, SRCCOPY );
	BitmapDC.SelectObject( old_bitmap );
  }
}


// Here is the bitmap for off-screen drawing...
//
// We share one bitmap with all views, we can get anyway with static members
// because we are not multi-thread
CBitmap		CTinyCadView::m_bitmap;
int			CTinyCadView::m_bitmap_width = 0;
int			CTinyCadView::m_bitmap_height = 0;
int			CTinyCadView::m_max_bitmap_size = 2048 * 2048;


// Create a bitmap for off-screen drawing...
bool CTinyCadView::CreateBitmap( CDC &dc, int width, int height )
{
	// Is there already a suitable bitmap?
	if (m_bitmap_width >= width && m_bitmap_height >= height)
	{
		return true;
	}


	// Is this beyond the maximum size we are willing to allocate?
	if (width * height > m_max_bitmap_size)
	{
		return false;
	}


	int bpp = dc.GetDeviceCaps( BITSPIXEL );
	if (bpp <= 16)
	{
		bpp = 16;
	}
	else
	{
		bpp = 24;
	}

	// Now try and create the bitmap...
	struct
	{
		BITMAPINFO bi;
		RGBQUAD bipal[3];
	} q;
	q.bi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER); 
	q.bi.bmiHeader.biWidth = width; 
	q.bi.bmiHeader.biHeight = height; 
	q.bi.bmiHeader.biPlanes = 1; 
	q.bi.bmiHeader.biBitCount = bpp;
	q.bi.bmiHeader.biCompression = bpp == 16 ? BI_BITFIELDS : BI_RGB ;
	q.bi.bmiHeader.biSizeImage = 0; 
	q.bi.bmiHeader.biXPelsPerMeter = 0; 
	q.bi.bmiHeader.biYPelsPerMeter = 0; 
	q.bi.bmiHeader.biClrUsed = bpp == 16 ? 3 : 0; 
	q.bi.bmiHeader.biClrImportant = 0; 
	q.bi.bmiColors[0].rgbRed = 0;
	q.bi.bmiColors[0].rgbGreen = 0;
	q.bi.bmiColors[0].rgbBlue = 0;
	q.bi.bmiColors[0].rgbReserved = 0;

	 // Set up the 5-6-5 bit masks
	if (bpp == 16)
	{
		((DWORD*)(q.bi.bmiColors))[0] = (WORD)(0x1F << 11); //make sure that RGQQUAD array is after the q.bi struct
		((DWORD*)(q.bi.bmiColors))[1] = (WORD)(0x3F << 5);  //otherwise you will get an access violation
		((DWORD*)(q.bi.bmiColors))[2] = (WORD)(0x1F << 0);
	}
 

	void *bits;
	HBITMAP hb = CreateDIBSection(dc.m_hDC, &q.bi, DIB_RGB_COLORS, &bits, NULL, 0 ); 

	if (!hb)
	{
		// Probably not enough memory...
		return false;
	}


	// Do we need to destroy the old bitmap?
	if (m_bitmap.m_hObject)
	{
		m_bitmap.DeleteObject();
	}

	m_bitmap.Attach( hb );
	m_bitmap_width = width;
	m_bitmap_height = height;

	return true;
}






// The Zoom function
// OnSetZoom:
// Sets the zoom from a menu selection
void CTinyCadView::ChangeZoom(LONG NewZoom)
{
  theTransform.SetZoom(NewZoom);
}


void CTinyCadView::SetZoom(LONG NewZoom)
{
  ChangeZoom(NewZoom);
  SetScroll(theTransform.GetOrigin().x,theTransform.GetOrigin().y);
  Invalidate();
}



// Track the size of the window
void CTinyCadView::OnSize(UINT a,int cx,int cy)
{
	CView::OnSize(a,cx,cy);

	if (IsWindowVisible())
	{
		((CTinyCadApp*)AfxGetApp())->SetMdiMaximize( GetParentFrame()->IsZoomed() != 0 );	
	}


	if (m_pDocument == NULL)
		return;

	CRect nSize;
	GetClientRect(nSize);
	if (vRuler != NULL)
		vRuler->OnNewSize(nSize);
	if (hRuler != NULL)
		hRuler->OnNewSize(nSize);

	if (GetCurrentDocument())
	{
		SetScroll(theTransform.GetOrigin().x,theTransform.GetOrigin().y);
	}

}


// The scroll bar functions:
// These control the horizontal and vertical scrolling of the window
// (uses OnSize to track the window's size to stop the user being able to place text off the max size)
static LONG oldZoom = 100;
void CTinyCadView::SetScroll(int NewX,int NewY)
{
  CRect rect;
  int px,py;

  // How big is the current window?
  GetClientRect(rect);

  // Convert region into internal co-ords
  px=theTransform.intDeScale(rect.right);
  py=theTransform.intDeScale(rect.bottom);



  // Allow a 10% overlap
  int Xlap=GetCurrentDocument()->GetDetails().GetPageBoundsAsPoint().x/10;
  int Ylap=GetCurrentDocument()->GetDetails().GetPageBoundsAsPoint().y/10;

  // Now does this fit?
  if (px+NewX>GetCurrentDocument()->GetDetails().GetPageBoundsAsPoint().x+Xlap)
	NewX=GetCurrentDocument()->GetDetails().GetPageBoundsAsPoint().x+Xlap-px;
  if (py+NewY>GetCurrentDocument()->GetDetails().GetPageBoundsAsPoint().y+Ylap)
	NewY=GetCurrentDocument()->GetDetails().GetPageBoundsAsPoint().y+Ylap-py;
  if (NewX<-Xlap)
	NewX=-Xlap;
  if (NewY<-Ylap)
	NewY=-Ylap;

  if (theTransform.GetOrigin()!=CPoint(NewX,NewY) || oldZoom!=theTransform.GetZoom()) 
  {
  	theTransform.SetOriginX(NewX);
  	theTransform.SetOriginY(NewY);

	SCROLLINFO si;
	si.cbSize = sizeof( SCROLLINFO );
	si.nMin = -Xlap;
	si.nMax = GetCurrentDocument()->GetDetails().GetPageBoundsAsPoint().x+Xlap;
	si.nPage = theTransform.intDeScale(rect.Width());
	si.nPos = theTransform.GetOrigin().x;
	si.fMask = SIF_PAGE | SIF_POS | SIF_RANGE | SIF_DISABLENOSCROLL;
	SetScrollInfo( SB_HORZ, &si, TRUE );

	si.cbSize = sizeof( SCROLLINFO );
	si.nMin = -Ylap;
	si.nMax = GetCurrentDocument()->GetDetails().GetPageBoundsAsPoint().y+Ylap;
	si.nPage = theTransform.intDeScale(rect.Height());;
	si.nPos = theTransform.GetOrigin().y;
	si.fMask = SIF_PAGE | SIF_POS | SIF_RANGE | SIF_DISABLENOSCROLL;
	SetScrollInfo( SB_VERT, &si, TRUE );


	if (hRuler != NULL)
		hRuler->OnNewTransform(theTransform);
	if (vRuler != NULL)
		vRuler->OnNewTransform(theTransform);
	Invalidate();
  }

  oldZoom=theTransform.GetZoom();
}

void CTinyCadView::SetScrollCentre(CPoint c)
{
  CRect rect;
  CPoint p;

  GetClientRect(rect);
  p=theTransform.DeScale(GetCurrentDocument()->m_snap,CPoint(rect.right/2,rect.bottom/2));
  SetScroll(theTransform.GetOrigin().x+c.x-p.x,theTransform.GetOrigin().y+c.y-p.y);  
}



// The message handlers for the VScroll and HScroll messages
void CTinyCadView::OnHScroll( UINT wParam, UINT pos, CScrollBar* )
{
  CRect rect;
  int moveFast,moveSlow;
  int x = theTransform.GetOrigin().x;
  int y = theTransform.GetOrigin().y;

  GetClientRect(rect);
  int Width = theTransform.intDeScale(rect.right);
  moveFast=(Width<<4)/20;
  moveSlow=(Width*GetCurrentDocument()->GetOptions()->GetScrollBar())/100;

  switch (wParam) {
	case SB_LINEUP:
		SetScroll(x-moveSlow,y);
		break;
	case SB_PAGEUP:
		SetScroll(x-moveFast,y);
		break;
	case SB_LINEDOWN:
		SetScroll(x+moveSlow,y);
		break;
	case SB_PAGEDOWN:
		SetScroll(x+moveFast,y);
		break;
	case SB_THUMBTRACK:
	case SB_THUMBPOSITION:
		SetScroll(pos,y);
		break;
  }
}


// The message handlers for the VScroll and HScroll messages
void CTinyCadView::OnVScroll( UINT wParam, UINT pos, CScrollBar* )
{
  CRect rect;
  int moveFast,moveSlow;
  int x = theTransform.GetOrigin().x;
  int y = theTransform.GetOrigin().y;


  GetClientRect(rect);
  int Width = theTransform.intDeScale(rect.bottom);
  moveFast=(Width<<4)/20;
  moveSlow=(Width*GetCurrentDocument()->GetOptions()->GetScrollBar())/100;

  switch (wParam) {
	case SB_LINEUP:
		SetScroll(x,y-moveSlow);
		break;
	case SB_PAGEUP:
		SetScroll(x,y-moveFast);
		break;
	case SB_LINEDOWN:
		SetScroll(x,y+moveSlow);
		break;
	case SB_PAGEDOWN:
		SetScroll(x,y+moveFast);
		break;
	case SB_THUMBTRACK:
	case SB_THUMBPOSITION:
		SetScroll(x,pos);
		break;
  }
}


