/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include "TinyCadView.h"
#include "special.h"
#include "library.h"
#include "TinyCad.h"

////// The auto anotate special function //////

AnotateSetup theASetup;

// Auto anotate the design
void CTinyCadView::OnSpecialAnotate()
{
  CDlgAnotateBox theDialog(this,theASetup);

  // Get rid of any drawing tool
  GetCurrentDocument()->SelectObject(new CDrawEditItem(GetCurrentDocument()));

  // Do the dialog
  int action = theDialog.DoModal();
  
  if (action == IDC_REF_PAINTER)
  {
	  theASetup = theDialog.v;
	  GetCurrentDocument()->SelectObject(new CDrawRefPainter(GetCurrentDocument(), theASetup.startval ));
	  return;
  }
  else if (action !=IDOK)
  {
	return;
  }

  theASetup = theDialog.v;

  // Set the busy icon
  SetCursor( AfxGetApp()->LoadStandardCursor( IDC_WAIT ) );
  

  // Now add/remove references
  CDrawMethod *thisMethod;
  CSymbolRecord *thisSymbol;
  int value=0;
  int part=0;
  BOOL IsSet,IsMatch,MissingSymbols=FALSE;
  

  for (int i = 0; i < 2; i++)
  {
		drawingIterator it = GetCurrentDocument()->GetDrawingBegin();
		while (it != GetCurrentDocument()->GetDrawingEnd()) 
		{
			CDrawingObject *pointer = *it;
			// Look for method objects
			if (pointer->GetType() == xMethodEx3) 
			{
				thisMethod = (CDrawMethod *)pointer;
				thisSymbol = thisMethod->GetSymbolData();

				// If there is no symbol then cannot modify this symbol!
				if (thisMethod->IsNoSymbol()) 
				{
					MissingSymbols = TRUE;
					++ it;
					continue;
				}

				// Has this symbol got a reference?
				IsSet   = thisMethod->HasRef();

				switch (theASetup.reference)
				{
				case 0:		// All references
					IsMatch = TRUE;
					break;
				case 1:		// Un-numbered references
					IsMatch = !thisMethod->HasRef();
					break;
				case 2:		// References that match...
					IsMatch = theASetup.matchval == thisSymbol->reference;
					break;
				}

				if (IsMatch)
				{

					// First pass  - we remove references if necessary,
					// Second pass - we add refences back in...
					//
					if (i == 0)
					{
						// Remove any matching references (if necessary)
						if (IsSet && (theASetup.value!=1 || thisMethod->GetRefVal()>=theASetup.startval) ) 
						{
							thisMethod->RemoveReference();
						}
					}
					else
					{
						// Now add back any references
						if (theASetup.action == 0) 
						{
							if (theASetup.reference != 1)
							{
								value = (theASetup.value == 0) ? 1 : theASetup.startval;
							}
							thisMethod->AddReference( value );
						}
					}
				}
			}

			++ it;
		}
	}

  // Where there any missing symbols?
  if (MissingSymbols)
	Message(IDS_MISSMETH,MB_ICONEXCLAMATION);


  // Restore the correct cursor
  SetCursor( AfxGetApp()->LoadStandardCursor( IDC_ARROW ) );


  // Ensure the window is re-drawn
  GetCurrentDocument()->SetModifiedFlag( TRUE );
  Invalidate();  
}



// The Anotate Setup values class
AnotateSetup::AnotateSetup()
{
  theASetup.action=0;
  theASetup.reference=0;
  theASetup.value=0;
  theASetup.matchval="U?";
  theASetup.startval=1;
}





////// The Parts List (Bill of Materials) special function //////


#define PARTSLIST_PAD	25

// Increment the xpos counter
void XInc(FILE *theFile,int &Xpos,int amount)
{
	Xpos += amount;

	if (Xpos >=PARTSLIST_PAD) 
	{
		fprintf(theFile,"\n");
		Xpos = amount;
	}
}

class BomSort
{
public:
	bool operator()( const CDrawMethod *a, const CDrawMethod *b ) const
	{
		if (a->GetName() == b->GetName())
		{
			return a->GetRef() < b->GetRef();
		}

		return a->GetName() < b->GetName();
	}
};

void CTinyCadView::OnSpecialBom()
{
  // Get rid of any drawing tool
  GetCurrentDocument()->SelectObject(new CDrawEditItem(GetCurrentDocument()));

  // Get the file name for the parts list
  OPENFILENAME ofn;
  char szFile[256], szFileTitle[256];

  strcpy(szFile,GetDocument()->GetPathName());
  for (int i=0;szFile[i]!=0 && szFile[i]!='.';i++) ;
  strcpy(szFile+i, ".pts");

  strcpy(szFileTitle,szFile);

  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner = this->m_hWnd;
  ofn.lpstrFilter = "Parts List (*.pts)\0*.pts\0All files (*.*)\0*.*\0\0";
  ofn.lpstrCustomFilter = NULL;
  ofn.nMaxCustFilter = 0;
  ofn.nFilterIndex = 1;
  ofn.lpstrFile = szFile;
  ofn.nMaxFile = sizeof(szFile);
  ofn.lpstrFileTitle = szFileTitle;
  ofn.nMaxFileTitle = sizeof(szFileTitle);
  ofn.lpstrInitialDir = NULL;
  ofn.lpstrTitle = NULL;
  ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST  | OFN_NOCHANGEDIR;
  ofn.nFileOffset = 0;
  ofn.nFileExtension = 0;
  ofn.lpstrDefExt = "pts";
  ofn.lpstrTitle = "Create Parts List";

  if (!GetSaveFileName(&ofn))
      	return;


  FILE *theFile = fopen(szFile,"w");
  if (theFile == NULL) {
	Message(IDS_CANNOTOPEN);
	return;
  }

  // Set the Busy icon
  SetCursor( AfxGetApp()->LoadStandardCursor( IDC_WAIT ) );


  // Build the set of sorted methods
  std::set<CDrawMethod*,BomSort> methods;
  drawingIterator it = GetCurrentDocument()->GetDrawingBegin();
  while (it != GetCurrentDocument()->GetDrawingEnd()) 
  {
	CDrawingObject *ObjPtr = *it;

	if (ObjPtr->GetType() == xMethodEx3) 
	{
		CDrawMethod *thisMethod = (CDrawMethod*)ObjPtr;
		methods.insert( thisMethod );
	}

	++ it;
  }

  fprintf(theFile,"====+  Parts List for %s  +====\n\n",(const char *)(GetDocument()->GetPathName()));

  BOOL MissingRef = FALSE;
  std::set<CDrawMethod*,BomSort>::iterator itx = methods.begin();
  CString LastRef = "";
  CString LastName = "";
  int Xpos = 0, peices=0;
  while (itx != methods.end()) 
  {
	CDrawMethod *thisMethod = *itx;
	CString name = thisMethod->GetName();

	if (name != LastName && LastName != "")
	{
		// Pad with spaces
		while (Xpos < PARTSLIST_PAD) 
		{
			fprintf(theFile," ");
			Xpos ++;
		}

		// Write the name
		fprintf(theFile,"   %3d   %s\n",peices,(const char *)LastName);
		Xpos = 0;
		peices = 0;
	}

	// Do not write out references more than once
	if (thisMethod->GetRef() != LastRef) 
	{
		if (Xpos!=PARTSLIST_PAD-1 && Xpos != 0) 
		{
			XInc(theFile,Xpos,1);
			fprintf(theFile,",");
		}
		XInc(theFile,Xpos,(thisMethod->GetRef()).GetLength());
		fprintf(theFile,"%s",(const char *)(thisMethod->GetRef()));
		peices ++;
	}

	if (thisMethod->GetRef()==(thisMethod->GetSymbolData())->reference)
		MissingRef = TRUE;

	LastRef = thisMethod->GetRef();
	LastName = name;

	++ itx;
  }


  if (peices > 0)
  {
	// Pad with spaces
	while (Xpos < PARTSLIST_PAD) 
	{
		fprintf(theFile," ");
		Xpos ++;
	}

	// Write the name
	fprintf(theFile,"   %3d   %s\n",peices,(const char *)LastName);
  }


  fclose(theFile);

  // Restore the normal cursor
  SetCursor( AfxGetApp()->LoadStandardCursor( IDC_ARROW ) );

  // Where there any errors?
  if (MissingRef)
	Message(IDS_MISSREF);
}




