/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __SYMBOL_H__
#define __SYMBOL_H__

#include <vector>

#include "DrawingObject.h"

class CLibraryStore;
class CTinyCadSymbolDoc;

enum SymbolFieldType
{
	default_show,
	default_hidden,
	always_hidden,
	extra_parameter
};


class CSymbolField
{
public:
	CString		field_name;
	CString		field_value;	// Not used!
	CString		field_default;
	CPoint		field_pos;		// Not used!
	

	SymbolFieldType	field_type;

	// Save this field
	void Save(CStream &);							// Save this symbol field into an archive
	void SaveXML(CXMLWriter &xml, bool simple);		// Save this symbol field into xml
	void Load(CStream &);							// Load this symbol field from an archive
	void LoadXML(CXMLReader &xml);					// Load this symbol field from xml
};

// Here is the data that is associated with
// all symbols
class CSymbolRecord
{
public:
	BOOL LoadSymbol( CTinyCadDoc *pDesign, CStream &theArchive, CPoint &tr, drawingCollection &drawing );
	void PostLoad(CTinyCadDoc *pDesign, CPoint &tr, drawingCollection &drawing);

public:
	// The constructor
	CSymbolRecord();

	CString			name;			// The name of this symbol
	CString			reference;		// The default reference
	CString			description;	// The description of this symbol
	SymbolFieldType	name_type;		// Do we show the name by default?
	SymbolFieldType	ref_type;		// Do we show the reference by default?
	DWORD			NameID;			// The ID of this name when stored in a database

	void SaveXML(CXMLWriter &xml);	// Save this symbol into an archive
	void LoadXML(CXMLReader &xml);	// Load this symbol from an archive


	// Here are the other fields we can have
	std::vector<CSymbolField>	fields;
	BOOL						fields_loaded;
};



class CDesignFileSymbol : public CSymbolRecord
{
protected:
	CTinyCadDoc*	  m_pDesign;
	drawingCollection m_method;
	CPoint			  m_Tr;

public:
	CDesignFileSymbol(CTinyCadDoc *pDesign, drawingCollection &method, CPoint tr );
	virtual ~CDesignFileSymbol();		// The Destructor
	
	DWORD			FilePos;			// The position in the file or database of this symbol
	BYTE 			ppp;				// Parts per package


	void SaveXML(CXMLWriter &xml);			// Save this symbol into an archive
	void Load(CTinyCadDoc *, CStream &);	// Load this symbol from an archive
	void LoadXML(CTinyCadDoc *, CXMLReader &xml);	// Load this symbol from an archive

	// Return the methods object
	virtual BOOL GetMethod( CPoint &tr, drawingCollection &drawing );

	CTinyCadDoc* GetDesign() { return m_pDesign; }
};


class CLibraryStoreNameSet;

class CLibraryStoreSymbol : public CSymbolRecord
{
public:
	CLibraryStoreNameSet*	m_pParent;
	int						m_index;

	CLibraryStoreSymbol();

	CDesignFileSymbol *GetDesignSymbol( CTinyCadDoc *pDesign );
};

typedef std::vector<CLibraryStoreSymbol>	recordCollection;


class CLibraryStoreNameSet
{
private:

	recordCollection	m_records;

public:

	CLibraryStore *lib;		// The library that this symbol comes from

	CLibraryStoreNameSet(CLibraryStore *NewLib = NULL);	// The constructor
	virtual ~CLibraryStoreNameSet();		// The Destructor

	CLibraryStoreSymbol&	GetRecord( int index );
	int						GetNumRecords();
	void					SetRecords( recordCollection& records );
	void					PushBackRecord( CLibraryStoreSymbol &r );
	BOOL					GetMethod( CTinyCadDoc *pDesign, CPoint &tr, drawingCollection &drawing );	

	BYTE 			ppp;			// Parts per package
	int				orientation;	// The default orientation of the symbol
	DWORD			FilePos;		// The position in the file or database of this symbol

	// Make a blank symbol record
	void			Blank();

	// Save this NameSet as an XML export
	void SaveXML( CXMLWriter &xml );

	// Load this NameSet from an XML file
	void LoadXML(CTinyCadDoc *pDesign, drawingCollection &drawing, CXMLReader &xml);
	
	void Save(CStream &);		// Save this symbol into an archive
	void Load(CStream &);		// Load this symbol from an archive
	void OldLoad1(CStream &);	// Load the old versions of symbol
	void OldLoad2(CStream &);	// Load the old versions of symbol
	void OldLoad3(CStream &);	// Load the old versions of symbol

	// Return the methods object
	CString GetLibName();

	CDesignFileSymbol *GetDesignSymbol( CTinyCadDoc *pDesign, int index );
};




#endif


