/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include "TinyCad.h"
#include "MainFrm.h"
#include "ChildFrm.h"
#include "TinyCadView.h"
#include "Registry.h"
#include "LibraryView.h"
#include "LibraryDoc.h"
#include "TinyCadMultiSymbolDoc.h"
#include "TinyCadMultiDoc.h"
#include "LibraryDb.h"
#include "LibraryCollection.h"
#include "DlgLibraryBox.h"
#include "DlgAbout.h"
#include "AutoSave.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//*************************************************************************
//*                                                                       *
//*                  Main application class                               *
//*                                                                       *
//*************************************************************************

CTinyCadApp::CTinyCadApp()
{
}

CTinyCadApp::~CTinyCadApp()
{
	CLibraryCollection::Clear();
	delete m_pRegistry;
}



BEGIN_MESSAGE_MAP(CTinyCadApp, CWinApp)
	//{{AFX_MSG_MAP(CTinyCadApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(IDM_LIBLIB, OnLibLib)
	ON_COMMAND(ID_HELP_GOTOTINYCADWEBSITE, OnHelpGototinycadwebsite)
	ON_COMMAND(ID_HELP_HELP, OnHelpHelp)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

//=========================================================================
//== init class variables                                                ==
//=========================================================================
CTinyCadApp CTinyCadApp::m_oInstance;
CMultiDocTemplate* CTinyCadApp::m_pDocTemplate	= NULL;
CMultiDocTemplate* CTinyCadApp::m_pLibTemplate	= NULL;
bool				CTinyCadApp::m_Maximize		= false;
bool				CTinyCadApp::m_MdiMaximize	= false;
bool				CTinyCadApp::m_PrintBandW	= false;
bool				CTinyCadApp::m_ShowPosition	= false;
int					CTinyCadApp::m_PrintScale	= 100;

//=========================================================================
//== ctor/dtor/initializing                                              ==
//=========================================================================
BOOL CTinyCadApp::InitInstance()
{
	if (!AfxSocketInit())
	{
		AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
		return FALSE;
	}

	AfxEnableControlContainer();

	// Force system to use DA0 3.60 not 3.50.
	AfxGetModuleState()->m_dwVersion = 0x0601;

	// Standard initialization
#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	// Change the registry key under which our settings are stored.
	// TODO: You should modify this string to be something appropriate
	// such as the name of your company or organization.
	SetRegistryKey( CTinyCadApp::GetName() );
	m_pRegistry = new CRegistry();

	// Load standard INI file options (including MRU)
	LoadStdProfileSettings(6);

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.
	m_pDocTemplate = new CMultiDocTemplate(
		IDR_TCADTYPE,
		RUNTIME_CLASS(CTinyCadMultiDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CTinyCadView));
	AddDocTemplate(m_pDocTemplate);

	m_pLibTemplate = new CMultiDocTemplate(
		IDR_LIBTYPE,
		RUNTIME_CLASS(CLibraryDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CLibraryView));
	AddDocTemplate(m_pLibTemplate);

	// Read the registry
	ReadRegistry();

	// create main MDI Frame window
	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;

	// Enable drag/drop open
	m_pMainWnd->DragAcceptFiles();

	// Enable DDE Execute open
	EnableShellOpen();
	RegisterShellFileTypes(TRUE);

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

	if (m_Maximize && m_nCmdShow == 1)
	{
		m_nCmdShow = SW_SHOWMAXIMIZED;
	}

	// The main window has been initialized, so show and update it.
	pMainFrame->ShowWindow(m_nCmdShow);
	pMainFrame->UpdateWindow();

	CAutoSave::Start();

	return TRUE;
}
//-------------------------------------------------------------------------

//=========================================================================
//== accessor                                                            ==
//=========================================================================

//-------------------------------------------------------------------------
bool CTinyCadApp::IsWinNT()
{
	bool			bReturn = true;
	OSVERSIONINFO	oVersion;

	oVersion.dwOSVersionInfoSize = sizeof( OSVERSIONINFO );
	GetVersionEx( &oVersion );

	bReturn = (oVersion.dwPlatformId == VER_PLATFORM_WIN32_NT);

	return bReturn;
}
//-------------------------------------------------------------------------
CString CTinyCadApp::GetVersion()
{
	CString				sReturn;
	char				szModulePath[MAX_PATH];
	DWORD				dwSize;
	DWORD				dwZero;
	char*				pBuffer;
	VS_FIXEDFILEINFO*	pFixedInfo;
	UINT				uVersionLen;


	GetModuleFileName(NULL, szModulePath, MAX_PATH);
	dwSize = GetFileVersionInfoSize(szModulePath, &dwZero);

	if( dwSize > 0 )
	{
		pBuffer = (char *)malloc(dwSize * sizeof(char));

		if( pBuffer )
		{
			GetFileVersionInfo(szModulePath, dwZero, dwSize, pBuffer);
			VerQueryValue(pBuffer, "\\", (void**) &pFixedInfo, (UINT*)
			&uVersionLen);

			sReturn.Format( "version %u.%02u.%02u", HIWORD(pFixedInfo->dwProductVersionMS),
										LOWORD(pFixedInfo->dwProductVersionMS),
										HIWORD(pFixedInfo->dwProductVersionLS));

			free( pBuffer );
		}
	}

	return sReturn;
}
//-------------------------------------------------------------------------
CString CTinyCadApp::GetName()
{
	return "TinyCAD";
}
//-------------------------------------------------------------------------
CString CTinyCadApp::GetMainDir()
{
	CString sReturn;
	TCHAR 	theBuffer[1024];
	DWORD 	theBytes = GetModuleFileName(NULL,theBuffer,sizeof(theBuffer) - 1);

	if( theBytes != 0 )
	{
		TCHAR* thePtr = theBuffer + theBytes;

		while (thePtr >= theBuffer && *thePtr != _TCHAR('\\'))
  	{
			thePtr--;
		}
		*thePtr = _TCHAR('\0');

		sReturn = CString( theBuffer ) + "\\";
  }

	return sReturn;
}
//-------------------------------------------------------------------------
bool CTinyCadApp::GetMdiMaximize()
{
	return m_MdiMaximize;
}
//-------------------------------------------------------------------------
bool CTinyCadApp::GetPrintBandW()
{
	return m_PrintBandW;
}
//-------------------------------------------------------------------------
int CTinyCadApp::GetPrintScale()
{
	return m_PrintScale;
}
//-------------------------------------------------------------------------

//=========================================================================
//== mutator                                                             ==
//=========================================================================
	
//-------------------------------------------------------------------------
// Read the registry
void CTinyCadApp::ReadRegistry()
{
	// Is there a list of libraries in the registry?
	CStringList*	colLibs = Registry::GetStringList( "Libraries" );

	// One single library
	CString			sLibName;

	CLibraryStore*	oLib;


	// Iterate through the list in head-to-tail order.
	for( POSITION pos = colLibs->GetHeadPosition(); pos != NULL; )
	{
		sLibName = colLibs->GetNext( pos );

		// Is this a new file library or an old library type?
		FILE* f = fopen( sLibName + ".idx", "rb" );

		if( f )
		{
			fclose(f);
			oLib = new CLibraryFile;
		}
		else
		{
			oLib = new CLibraryDb;
		}
		
		oLib->Attach( sLibName );
		CLibraryCollection::Add( oLib );
	}

	delete colLibs;

	m_Maximize		= Registry::GetBool( "Maximize",		false );
	m_MdiMaximize	= Registry::GetBool( "MdiMaximize",		false );
	m_ShowPosition	= Registry::GetBool( "ShowPosition",	false );
	m_PrintBandW	= Registry::GetBool( "PrintBandW",		false );
	m_PrintScale	= Registry::GetInt(	 "PrintScale",		100 );
}
//-------------------------------------------------------------------------
void CTinyCadApp::SetMaximize( bool bHasMaxScreen )
{
	m_Maximize = bHasMaxScreen;
	Registry::Set( "Maximize", m_Maximize );
}
//-------------------------------------------------------------------------
void CTinyCadApp::SetMdiMaximize( bool bHasMaxMdi )
{
	m_MdiMaximize = bHasMaxMdi;
	Registry::Set( "MdiMaximize", m_MdiMaximize );
}
//-------------------------------------------------------------------------
void CTinyCadApp::SetPrintBandW( bool m )
{
	m_PrintBandW = m;
	Registry::Set( "PrintBandW", m_PrintBandW );
}
//-------------------------------------------------------------------------
void CTinyCadApp::SetPrintScale( int m )
{
	m_PrintScale = m;
	Registry::Set( "PrintScale", m_PrintScale );
}
//-------------------------------------------------------------------------

//=========================================================================
//==    manage views                                                     ==
//=========================================================================

//-------------------------------------------------------------------------
// Edit a library using the doc/view
void CTinyCadApp::EditSymbol(CLibraryStore* pLib, CLibraryStoreNameSet &symbol)
{
	CTinyCadMultiSymbolDoc *pMulti = new CTinyCadMultiSymbolDoc( pLib, symbol );

	CFrameWnd *pFrame = m_pDocTemplate->CreateNewFrame( pMulti, NULL );
	m_pDocTemplate->InitialUpdateFrame( pFrame, pMulti, TRUE );
}
//-------------------------------------------------------------------------
// Edit a library using the doc/view
void CTinyCadApp::EditLibrary(CLibraryStore* pLib)
{
	// Is this library already being edited?
	POSITION p = m_pLibTemplate->GetFirstDocPosition();
	while (p != NULL)
	{
		CLibraryDoc *t = static_cast<CLibraryDoc*>(m_pLibTemplate->GetNextDoc(p));
		if (t->IsLibInUse( pLib ))
		{
			POSITION v = t->GetFirstViewPosition();
			
			// Active it's views
		   while (v != NULL)
		   {
			  CView* pView = t->GetNextView(v);
			  pView->GetParentFrame()->BringWindowToTop();
		   }   

		   return;
		}		
	}

	CLibraryDoc *pDoc = static_cast<CLibraryDoc *>(m_pLibTemplate->CreateNewDocument());

	if (pDoc != NULL)
	{
		pDoc->setLibrary( pLib );
		CFrameWnd *pFrame = m_pLibTemplate->CreateNewFrame( pDoc, NULL );
		m_pLibTemplate->InitialUpdateFrame( pFrame, pDoc, TRUE );
	}
}
//-------------------------------------------------------------------------
void CTinyCadApp::ResetAllSymbols()
{
	static_cast<CMainFrame*>(AfxGetMainWnd())->ResetAllSymbols();
	
	POSITION p = m_pLibTemplate->GetFirstDocPosition();

	while (p != NULL)
	{
		CLibraryDoc *t = static_cast<CLibraryDoc*>(m_pLibTemplate->GetNextDoc(p));
		t->ResetSymbols();
	}
}
//-------------------------------------------------------------------------
void CTinyCadApp::SaveAll()
{
	// Save a backup copy of all of the files so far
	POSITION p = m_pDocTemplate->GetFirstDocPosition();

	while (p != NULL)
	{
		CTinyCadMultiDoc *t = static_cast<CTinyCadMultiDoc*>(m_pDocTemplate->GetNextDoc(p));
		
		// Force an auto-save...
		t->AutoSave();
	}

}
//-------------------------------------------------------------------------
bool CTinyCadApp::IsLibInUse( CLibraryStore* pLib )
{
	POSITION p = m_pDocTemplate->GetFirstDocPosition();

	while (p != NULL)
	{
		CTinyCadMultiDoc *t = static_cast<CTinyCadMultiDoc*>(m_pDocTemplate->GetNextDoc(p));
		if (t->IsLibInUse( pLib ))
		{
			return true;
		}
	}

	p = m_pLibTemplate->GetFirstDocPosition();
	while (p != NULL)
	{
		CLibraryDoc *t = static_cast<CLibraryDoc*>(m_pLibTemplate->GetNextDoc(p));
		if (t->IsLibInUse( pLib ))
		{
			return true;
		}		
	}

	return false;
}
//-------------------------------------------------------------------------

//=========================================================================
//== Message handler                                                     ==
//=========================================================================

//-------------------------------------------------------------------------
// This is the idle time processing
BOOL CTinyCadApp::OnIdle( LONG nCount )
{
  	CWinApp::OnIdle( nCount );

  	if( nCount == 0 )
  	{
		CLibraryCollection::DoIdle();
  	}

  	return FALSE;
}
//-------------------------------------------------------------------------
void CTinyCadApp::OnLibLib()
{
	CDlgLibraryBox( AfxGetMainWnd() ).DoModal();
}
//-------------------------------------------------------------------------
void CTinyCadApp::OnHelpGototinycadwebsite() 
{
	// Open a browser for our web site
	ShellExecute( AfxGetMainWnd()->m_hWnd, "open", "http://tinycad.sourceforge.net", NULL, NULL, SW_SHOWNORMAL );	
}


//-------------------------------------------------------------------------
void CTinyCadApp::OnAppAbout()
{
	CDlgAbout().DoModal();
}
//-------------------------------------------------------------------------
