/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#if !defined(AFX_TINYCAD_H__AC021B43_46B6_4350_8BEB_1EC76FE565B5__INCLUDED_)
#define AFX_TINYCAD_H__AC021B43_46B6_4350_8BEB_1EC76FE565B5__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

class CRegistry;

#include "resource.h"
#include "LibraryStore.h"


//*************************************************************************
//*                                                                       *
//*                  Main application class                               *
//*                                                                       *
//*************************************************************************
class CTinyCadApp : public CWinApp
{
private:

	CRegistry* m_pRegistry;

	//=====================================================================
	//== class variables                                                 ==
	//=====================================================================
	//-- singleton instance of this class
	private: static CTinyCadApp			m_oInstance;

	// The document template for all TinyCadDoc types
	private: static CMultiDocTemplate* 	m_pDocTemplate;

	// The document template for library viewing/editing
	private: static CMultiDocTemplate*	m_pLibTemplate;

	private: static bool				m_Maximize;
	private: static bool				m_MdiMaximize;
	private: static bool				m_PrintBandW;
	private: static bool				m_ShowPosition;
	private: static int					m_PrintScale;

	//=====================================================================
	//== ctor/dtor/initializing                                          ==
	//=====================================================================
	virtual BOOL 				InitInstance();

public:
	CTinyCadApp();
	virtual ~CTinyCadApp();

	//=====================================================================
	//== accessor                                                        ==
	//=====================================================================
	public: static bool IsWinNT();
	public: static CString				GetVersion();
	public: static CString				GetName();
	public: static CString 				GetMainDir();
	public: bool						GetMdiMaximize();
	public: bool						GetPrintBandW();
	public: int							GetPrintScale();

	//=====================================================================
	//== mutator                                                         ==
	//=====================================================================
	//-- Read the settings from the registry
	private: void						ReadRegistry();

	public:	static	void 				SetMaximize( bool bHasMaxScreen );
	public:void SetMdiMaximize( bool bHasMaxMdi );
	public:void SetPrintBandW( bool bHasMaxScreen );
	public:void SetPrintScale( int m );

	//=====================================================================
	//==  manage views                                                   ==
	//=====================================================================
	public:static	void				EditSymbol( CLibraryStore* pLib, CLibraryStoreNameSet& symbol );
	public:static	void				EditLibrary( CLibraryStore* pLib );
	public:static	void				ResetAllSymbols();
	public:static	void				SaveAll();
	public:static	bool				IsLibInUse( CLibraryStore* pLib );

	//=====================================================================
	//== Message handler                                                 ==
	//=====================================================================
	virtual BOOL OnIdle(LONG lCount);
	//{{AFX_MSG(CTinyCadApp)
	afx_msg void OnAppAbout();
	afx_msg void				OnLibLib();
	afx_msg void OnHelpGototinycadwebsite();
	afx_msg void OnHelpHelp();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};
//=========================================================================
extern int Message(int Resource,int Type = MB_ICONINFORMATION | MB_OK, const char *NameString = "");
extern CString NameLength(const char *,int);


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_TINYCAD_H__AC021B43_46B6_4350_8BEB_1EC76FE565B5__INCLUDED_)
