/*
 * Project:		TinyCAD program for schematic capture
 *				http://tinycad.sourceforge.net
 * Copyright:	 1994-2005 Matt Pyne
 * License:		Lesser GNU Public License 2.1 (LGPL)
 *				http://www.opensource.org/licenses/lgpl-license.html
 */

#include "stdafx.h"
#include "TinyCad.h"
#include "TinyCadDoc.h"
#include "registry.h" 
#include "LineUtils.h"
#include <math.h>
#include "PngOutput.h"
#include "TinyCadMultiDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTinyCadDoc

#if 0
IMPLEMENT_DYNCREATE(CTinyCadDoc, CDocument)

BEGIN_MESSAGE_MAP(CTinyCadDoc, CDocument)
	//{{AFX_MSG_MAP(CTinyCadDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif

/////////////////////////////////////////////////////////////////////////////
// CTinyCadDoc construction/destruction

CTinyCadDoc::CTinyCadDoc(CMultiSheetDoc *pParent)
{

  theOptions.Init( this );

  m_pParent = pParent;
  selectable = NULL;
  m_undo_level = 0;
  m_change_set = FALSE;
  m_InUndoAddAction = FALSE;
	edit				= NULL;
  NameDir = 1;
  PinDir = 1;
  part = 0;

  // Now select a drawing object
	SelectObject( new CDrawEditItem(this) );

	// Attempt to load the defaults for the grid
	m_snap.SetAccurateGrid( Registry::GetInt("GridSpacing", NormalGrid) );
	m_snap.SetGridSnap( Registry::GetBool("GridSnap", TRUE) );
}

CTinyCadDoc::~CTinyCadDoc()
{
	Registry::Set("GridSpacing",m_snap.GetAccurateGrid());
	Registry::Set("GridSnap",m_snap.GetGridSnap());

	// Remove any editing tool
	if( edit )
	{
		edit->EndEdit();
		delete edit;
	}

	// Unselect all items
	UnSelect();

	// Now delete the contents of this document
	for( drawingIterator i = GetDrawingBegin(); i != GetDrawingEnd(); i++ ) 
	{
		CDrawingObject *pointer = *i;
		delete pointer;
	}
	
	m_drawing.clear();

	// Remove the undo/redo list
	m_undo_level = 0;
	FlushRedo();
}

#if 0
BOOL CTinyCadDoc::OnNewDocument()
{
	BOOL bReturn = super::OnNewDocument();

	if( bReturn )
	{
		GetDetails().Reset();

  SetPart(0);

  SelectObject(new CDrawEditItem(this));
  UnSelect();

	// Remove all objects from memory
		
		for( drawingIterator i = GetDrawingBegin(); i != GetDrawingEnd(); i++ ) 
	{
			CDrawingObject* pointer = *i;
		delete pointer;
	}
		m_drawing.clear();

    // Now delete the Redo List
	m_undo_level = 0;
	m_change_set = FALSE;
	FlushRedo();
	m_undo_level = 0;
  NameDir = 1;
  PinDir = 1;
  part = 0;
	}

	return bReturn;
}
#endif


// Iterate through the drawing
drawingIterator CTinyCadDoc::GetDrawingBegin()
{
	return m_drawing.begin();
}

drawingIterator CTinyCadDoc::GetDrawingEnd()
{
	return m_drawing.end();
}



/////////////////////////////////////////////////////////////////////////////
// CTinyCadDoc serialization


BOOL CTinyCadDoc::Import( CStream& ar )
{
	drawingCollection drawing;

	if (ReadFile(ar, FALSE, drawing) )
	{
		
		for( drawingCollection::iterator i = drawing.begin(); i != drawing.end(); i++ )
		{
			Add( *i );
		}
		UnSelect();
		SetModifiedFlag( FALSE );

		// Now select the objects
		UnSelect();
		
		for( drawingIterator j = drawing.begin(); j != drawing.end(); j++ )
		{
			Select( *j );
		}

		return TRUE;
	}

	return FALSE;
}


/////////////////////////////////////////////////////////////////////////////
// CTinyCadDoc commands


// Save as a PNG file
void CTinyCadDoc::SavePNG( const char *file_name, CDC &ref_dc, int scaling, bool bw )
{
	// Calculate the boundries
	CRect rect = CRect(0,0,0,0);

	if (IsSelected())
	{
		BOOL first = TRUE;

		selectIterator it = GetSelectBegin();
		while ( it != GetSelectEnd() ) 
		{
			CDrawingObject *pointer=*it;

			if (first) 
			{
				rect.top = pointer->a.y;
				rect.bottom = pointer->a.y;
				rect.left = pointer->a.x;
				rect.right = pointer->a.x;
				first = FALSE;
			}

			rect.top 	 = min(rect.top,min(pointer->a.y,pointer->b.y));
			rect.bottom = max(rect.bottom,max(pointer->a.y,pointer->b.y));
			rect.left 	 = min(rect.left,min(pointer->a.x,pointer->b.x));
			rect.right  = max(rect.right,max(pointer->a.x,pointer->b.x));

			++ it;
		}
	}
	else
	{
		rect = CRect( 0,0, GetDetails().m_szPage.cx, GetDetails().m_szPage.cy );
	}

	rect.InflateRect( 0,0, 10,10 );


	// Transform the origin so the design lies at 0,0
	Transform newTransform;
	newTransform.SetOrigin( CPoint( rect.left, rect.top ) );

	// Set the scaling
	newTransform.SetZoom( scaling );
	rect.right = (rect.right * scaling) / 100;
	rect.bottom = (rect.bottom * scaling) / 100;

	CPngOutput png;
	
	CBitmap *bitmap = png.CreateInputBitmap( rect.Width(), rect.Height(), ref_dc );


	CDC bitmap_dc;
	bitmap_dc.CreateCompatibleDC( &ref_dc );
	CBitmap *old_bitmap = bitmap_dc.SelectObject( bitmap );

	bitmap_dc.SelectStockObject( WHITE_BRUSH );
	bitmap_dc.SelectStockObject( WHITE_PEN );
	bitmap_dc.Rectangle( 0,0,rect.Width(), rect.Height() );

	// Create the DC and render...
	{
		Context dc(&bitmap_dc,newTransform);
		if (bw)
		{
			dc.SetBlack( TRUE );
		}

		if (IsSelected())
		{
			// Draw just the selected part
			
			for( selectIterator i = GetSelectBegin(); i != GetSelectEnd(); i++ ) 
			{
				(*i)->Paint(dc,draw_normal);
			}
		}
		else
		{
			// Draw the whole design into the metafile
			drawingIterator it = GetDrawingBegin();
			while (it != GetDrawingEnd()) 
			{
				(*it)->Paint(dc,draw_normal);
				++ it;
			}

			// Show the design 
			Display( dc );
		}

	}

	bitmap_dc.SelectObject( old_bitmap );


	// Now convert to 8bpp
	CBitmap output;
	png.QuantizeBuffer( output, ref_dc );

	// ... and save to a png
	png.Save( file_name );
}



// Convert this document into a metafile for pasting into
// other applications
HENHMETAFILE CTinyCadDoc::CreateMetafile( CDC &ref_dc )
{

	// Calculate the boundries
	CRect rect = CRect(0,0,0,0);

	if (IsSelected())
	{
		BOOL first = TRUE;

		selectIterator it = GetSelectBegin();
		while ( it != GetSelectEnd() ) 
		{
			CDrawingObject *pointer=*it;

			if (first) 
			{
				rect.top = pointer->a.y;
				rect.bottom = pointer->a.y;
				rect.left = pointer->a.x;
				rect.right = pointer->a.x;
				first = FALSE;
			}

			rect.top 	 = min(rect.top,min(pointer->a.y,pointer->b.y));
			rect.bottom = max(rect.bottom,max(pointer->a.y,pointer->b.y));
			rect.left 	 = min(rect.left,min(pointer->a.x,pointer->b.x));
			rect.right  = max(rect.right,max(pointer->a.x,pointer->b.x));

			++ it;
		}
	}
	else
	{
		rect = CRect( 0,0, GetDetails().m_szPage.cy, GetDetails().m_szPage.cy );
	}

	rect.InflateRect( 0,0, 10,10 );


	// Transform the origin so the design lies at 0,0
	Transform newTransform;


	int iWidthMM = ref_dc.GetDeviceCaps(HORZSIZE); 
	int iHeightMM = ref_dc.GetDeviceCaps(VERTSIZE); 
	int iWidthPels = ref_dc.GetDeviceCaps(HORZRES); 
	int iHeightPels = ref_dc.GetDeviceCaps(VERTRES); 
 
	// Convert client coordinates to .01-mm units. 
	// Use iWidthMM, iWidthPels, iHeightMM, and 
	// iHeightPels to determine the number of 
	// .01-millimeter units per pixel in the x- 
	//  and y-directions. 
 
	rect.left = (rect.left * iWidthMM * 100)/iWidthPels; 
	rect.top = (rect.top * iHeightMM * 100)/iHeightPels; 
	rect.right = (rect.right * iWidthMM * 100)/iWidthPels; 
	rect.bottom = (rect.bottom * iHeightMM * 100)/iHeightPels;


	// Create the metafile device context. 
	HDC hdcMeta = CreateEnhMetaFile(ref_dc.m_hDC,NULL, &rect, GetDetails().GetTitle() ); 
 
	if (!hdcMeta) 
	{
		AfxMessageBox("Cannot create enhanced metafile for the copy");
		return NULL;
	}

	CDC theMetaDC;
	theMetaDC.Attach( hdcMeta );

	// Now render into this dc
	newTransform.SetZoom( 100 );

	// Create the DC and render...
	{
		Context dc(&theMetaDC,newTransform);

		if (IsSelected())
		{
			// Draw just the selected part
			selectIterator it = GetSelectBegin();
			while ( it != GetSelectEnd() ) 
			{
				(*it)->Paint(dc,draw_normal);
				++ it;
			}
		}
		else
		{
			// Draw the whole design into the metafile
			for( drawingIterator i = GetDrawingBegin(); i != GetDrawingEnd(); i++ ) 
			{
				(*i)->Paint(dc,draw_normal);
			}

			// Show the design details
			Display( dc );
		}

	}

	theMetaDC.Detach();

	return CloseEnhMetaFile( hdcMeta );
}


// Make all symbol aware of the fact the libraries have changed
void CTinyCadDoc::ResetSymbols()
{
	theOptions.ResetSymbols();
}


// Tag all the resources being used by this design
void CTinyCadDoc::TagAllResources()
{
	for( drawingIterator i = GetDrawingBegin(); i != GetDrawingEnd(); i++ ) 
	{
		(*i)->TagResources();
	}
}

// Remove all errors from this design
void CTinyCadDoc::DeleteErrors()
{
  // Get rid of any drawing tool
  SelectObject(new CDrawEditItem(this));

	drawingIterator it = GetDrawingBegin();
	while (it != GetDrawingEnd()) 
	{
		drawingIterator current = it;
		++ it;
		
		CDrawingObject *pointer = *current;
		if (pointer->GetType()==xError) 
		{
			m_drawing.erase(current);
			delete pointer;
		}
	}

	Invalidate();
}


// Is this document editing a library?
bool CTinyCadDoc::IsLibInUse( CLibraryStore *lib )
{
	return false;
}

BOOL CTinyCadDoc::IsSelected()
{ 
	return m_selected.size() > 0;
}

selectIterator CTinyCadDoc::GetSelectBegin()
{
	return m_selected.begin();
}

selectIterator CTinyCadDoc::GetSelectEnd()
{
	return m_selected.end();
}

BOOL CTinyCadDoc::IsSelected( CDrawingObject * p )
{
	return m_selected.find( p ) != m_selected.end();
}
BOOL CTinyCadDoc::IsSingleItemSelected()
{
	return m_selected.size() == 1; 
}

CDrawingObject* CTinyCadDoc::GetSingleSelectedItem()
{
	if (m_selected.size() == 1)
	{
		return *(m_selected.begin());
	}
	else
	{
		return NULL;
	}
}



void CTinyCadDoc::Add( drawingCollection& drawing )
{
	for ( drawingCollection::iterator i = drawing.begin(); i != drawing.end(); ++ i)
	{
		(*i)->m_pDesign = this;
		m_drawing.push_back( *i );
	}
}

// Add a new object to this drawing
void CTinyCadDoc::Add(CDrawingObject *NewObject)
{
  // Is this a valid object?
  if (NewObject == NULL)
	return;

  // Store the object in the linked list
  m_drawing.insert( m_drawing.end(), NewObject );

  if (!m_InUndoAddAction)
  {
	MarkAdditionForUndo( NewObject );
  }

  // Set the dirty flag
  if (NewObject->GetType()!=xError)
  {
	SetModifiedFlag( TRUE );
  }
}


void CTinyCadDoc::BeginNewChangeSet()
{
	m_change_set = TRUE;
}


// Flush the Redo Buffer
void CTinyCadDoc::FlushRedo()
{
	for (int i = m_undo_level; i < m_undo.size(); i++)
	{
		CDocUndoSet &s = m_undo[ i ];
		CDocUndoSet::actionCollection::iterator act_it = s.m_actions.begin();

		while (act_it != s.m_actions.end())
		{
			delete (*act_it).m_object;
			++ act_it;
		}

		s.m_actions.resize( 0 );
	}

	m_undo.resize( m_undo_level + 1 );
}

// Undo the last action
void CTinyCadDoc::Undo()
{
	SetSelectable( NULL );
	BOOL action_taken = FALSE;

	// Is this possible?
	while (CanUndo() && !action_taken)
	{
		// Re-apply all of the changes we have done at this level
		CDocUndoSet &s = m_undo[ m_undo_level ];

		// Go through the list of action and undo each one in the reverse
		// order that they were applied
		CDocUndoSet::actionCollection::reverse_iterator act_it = s.m_actions.rbegin();

		while (act_it != s.m_actions.rend())
		{
			CDocUndoSet::CDocUndoAction &act = *act_it;


			// Look up this item from the index...
			drawingCollection::iterator it = m_drawing.begin();
			int index = act.m_index;
			while (index > 0 && it != m_drawing.end())
			{
				++ it;
				-- index;
			}


			act.m_object->Display();

			switch (act.m_action)
			{
			case CDocUndoSet::Deletion:
				// We must re-insert the deleted objects
				m_drawing.insert( it, Dup(act.m_object) );
				action_taken = TRUE;
				break;
			case CDocUndoSet::Addition:
				// We must remove the additions
				if (it != m_drawing.end())
				{
					delete *it;
					m_drawing.erase( it );
				}
				action_taken = TRUE;
				break;
			case CDocUndoSet::Change:
				// We convert the old objects into the new objects...
				{
					// Keep a copy for the redo...
					CDrawingObject *copy = Dup(*it);
					delete *it;

					copy->Display();
					*it = act.m_object;
					act.m_object = copy;
				}
				action_taken = TRUE;
				break;
			}

			++ act_it;
		}

		m_undo_level --;
	}
}

// Redo the last action
void CTinyCadDoc::Redo()
{
	SetSelectable( NULL );
	BOOL action_taken = FALSE;

	// Is this possible?
	while (CanRedo() && !action_taken)
	{


		m_undo_level ++;

		// Re-apply all of the changes we have done at this level
		CDocUndoSet &s = m_undo[ m_undo_level ];


		// Go through the list of action and redo each one 
		//
		CDocUndoSet::actionCollection::iterator act_it = s.m_actions.begin();

		while (act_it != s.m_actions.end())
		{
			CDocUndoSet::CDocUndoAction &act = *act_it;

			// Look up this item from the index...
			drawingCollection::iterator it = m_drawing.begin();
			int index = act.m_index;
			while (index > 0 && it != m_drawing.end())
			{
				++ it;
				-- index;
			}

			if (it != m_drawing.end())
			{
				(*it)->Display();
			}
			act.m_object->Display();

			switch (act.m_action)
			{
			case CDocUndoSet::Deletion:
				// We must re-delete the deleted objects
				delete *it;
				m_drawing.erase( it );
				action_taken = TRUE;
				break;
			case CDocUndoSet::Addition:
				// We must re-insert the additions
				m_drawing.insert( it, Dup(act.m_object) );
				action_taken = TRUE;
				break;
			case CDocUndoSet::Change:
				// We convert the old objects into the new objects...
				{
					// Keep a copy for the redo...
					CDrawingObject *copy = Dup(*it);
					delete *it;

					copy->Display();
					*it = act.m_object;
					act.m_object = copy;
				}
				action_taken = TRUE;
				break;
			}

			++ act_it;
		}
	}
}

// Make a duplicate of an object..
CDrawingObject*	CTinyCadDoc::Dup( CDrawingObject *p )
{
	// Now make a duplicate for the Undo/Redo list
	m_InUndoAddAction = TRUE;
	CDrawingObject *pNewObject = p->Store();
	m_InUndoAddAction = FALSE;
	m_drawing.pop_back();

	return pNewObject;
}

void CTinyCadDoc::AddUndoAction( CDocUndoSet::action action, CDrawingObject *index_object )
{
	// Do we need to increment the undo level?
	if (m_change_set)
	{
		// Increment the Undo position...
		m_undo_level ++;
		FlushRedo();
		m_change_set = FALSE;
	}


	if (m_undo_level >= m_undo.size())
	{
		m_undo.resize( m_undo_level + 1 );
	}
	CDocUndoSet &s = m_undo[ m_undo_level ];
    SetModifiedFlag( TRUE );


	// Look up this index...
	int index = 0;
	drawingIterator it = GetDrawingBegin();
	while (it != GetDrawingEnd()) 
	{
		if (*it == index_object)
		{
			break;
		}
		++ index;
		++ it;
	}



	// Now add this to the back of the undo action list
	CDocUndoSet::CDocUndoAction act;
	act.m_action = action;
	act.m_index = index;
	act.m_object = Dup(index_object);

	s.m_actions.push_back( act );
}

void CTinyCadDoc::MarkDeleteForUndo( CDrawingObject *pObject )
{
	AddUndoAction( CDocUndoSet::Deletion, pObject );
}

void CTinyCadDoc::MarkAdditionForUndo( CDrawingObject *pObject )
{
	AddUndoAction( CDocUndoSet::Addition, pObject );
}


void CTinyCadDoc::MarkChangeForUndo( CDrawingObject* pObject )
{
	AddUndoAction( CDocUndoSet::Change, pObject );
}

void CTinyCadDoc::MarkSelectChangeForUndo()
{
	selectIterator it = GetSelectBegin();
	while ( it != GetSelectEnd() ) 
	{
		MarkChangeForUndo(*it);
		++ it;
	}

}


// The object selection functions
// This selects objects in a box
// (We don't select construction objects)
void CTinyCadDoc::Select(CPoint p1,CPoint p2)
{
  int left=min(p1.x,p2.x);
  int right=max(p1.x,p2.x);
  int top=min(p1.y,p2.y);
  int bottom=max(p1.y,p2.y);

  UnSelect();

	drawingIterator it = GetDrawingBegin();
	while (it != GetDrawingEnd()) 
	{
		CDrawingObject *obj = *it;

		if (    obj->IsInside(left,right,top,bottom) 
			&& !obj->IsConstruction()
			&& 
			(obj->GetType() != xJunction || !GetOptions()->GetAutoJunc() ) )
		{
			obj->Display();
			Select( obj );
		} 
  		
		++ it;
	}
}


// Select an object
void CTinyCadDoc::Select(CDrawingObject *obj )
{
	m_selected.insert( obj );
}


// Unselect a single object
void CTinyCadDoc::UnSelect( CDrawingObject *p )
{
	selectIterator it = m_selected.find( p );
	if (it != m_selected.end())
	{
		m_selected.erase( it );
	}
}




// Unselect the objects
void CTinyCadDoc::UnSelect()
{
	// Get rid of any currently selected objects
	selectIterator it = GetSelectBegin();
	while ( it != GetSelectEnd() ) 
	{
		(*it)->Display();
		++ it;
	}

	m_selected.erase( m_selected.begin(), m_selected.end() );
}




// Change the Z-order of the selected objects
void CTinyCadDoc::BringToFront()
{
	if (!IsSelected())
		return;

	drawingCollection selectedObjects;
	drawingIterator it = GetDrawingBegin();
	while (it != GetDrawingEnd()) 
	{
		drawingIterator current = it;
		++ it;

		CDrawingObject *pointer = *current;
		if (IsSelected( pointer ))
		{
			MarkDeleteForUndo( pointer );
			m_drawing.erase( current );
			selectedObjects.push_back( pointer );
		}
	}


	// Now push them back on (in the right order)
	drawingCollection::iterator itx = selectedObjects.begin();
	while (itx != selectedObjects.end())
	{
		m_drawing.push_back( *itx );
		MarkAdditionForUndo( *itx );
		(*itx)->Display();
		++ itx;
	}

}


BOOL CTinyCadDoc::IsInDrawing( CDrawingObject *obj )
{
	drawingIterator it = GetDrawingBegin();
	while (it != GetDrawingEnd()) 
	{
		if (obj == *it)
		{
			return TRUE;
		}
		++ it;
	} 

	return FALSE;
}


// Change the Z-order of the selected objects
void CTinyCadDoc::SendToBack()
{
	if (!IsSelected())
		return;

	drawingCollection selectedObjects;
	drawingIterator it = GetDrawingBegin();
	while (it != GetDrawingEnd()) 
	{
		drawingIterator current = it;
		++ it;

		CDrawingObject *pointer = *current;
		if (IsSelected( pointer ))
		{
			MarkDeleteForUndo( pointer );
			m_drawing.erase( current );
			selectedObjects.push_back( pointer );
		}
	}

	// Now push them back on (in the right order)
	drawingCollection::reverse_iterator itx = selectedObjects.rbegin();
	while (itx != selectedObjects.rend())
	{
		m_drawing.push_front( *itx );
		MarkAdditionForUndo( *itx );
		(*itx)->Display();
		++ itx;
	}
}

// Remove an item from the drawing...
void CTinyCadDoc::Delete( drawingIterator it)
{
	CDrawingObject *pointer = *it;

	pointer->Display();
	if (pointer == GetSelectable())
	{
		SetSelectable( NULL );
	}
	UnSelect(pointer);

	MarkDeleteForUndo( *it );
	delete *it;
	m_drawing.erase( it );

	SetModifiedFlag( TRUE );
}



// Remove an item from the drawing...
void CTinyCadDoc::Delete( CDrawingObject *p )
{
	drawingIterator it = GetDrawingBegin();
	while (it != GetDrawingEnd()) 
	{
		drawingIterator current = it;

		CDrawingObject *pointer = *current;
		if (pointer == p)
		{
			Delete( current );
			break;
		}

		++ it;
	}
}


// Delete the selected objects!
void CTinyCadDoc::SelectDelete()
{
  if (!IsSelected())
	return;

    CJunctionUtils j( this );

	drawingIterator it = GetDrawingBegin();
	while (it != GetDrawingEnd()) 
	{
		drawingIterator current = it;
		++ it;

		CDrawingObject *pointer = *current;
		if (IsSelected( pointer ))
		{
			if (pointer == GetSelectable())
			{
				SetSelectable( NULL );
			}

			j.AddObjectToTodo( pointer );
			MarkDeleteForUndo( pointer );
			m_drawing.erase( current );
			pointer->Display();
			delete pointer;
		}
	}

	m_selected.erase( m_selected.begin(), m_selected.end() );

  // ... and perform the junction requirements...
  j.CheckTodoList( true );

  SetModifiedFlag( TRUE );
}




// Place the Selected objects into the Undo List in the original position
void CTinyCadDoc::SelectUndoMove(CPoint r)
{

  if (!IsSelected())
	return;

  	selectIterator it = GetSelectBegin();
	while ( it != GetSelectEnd() ) 
	{
		CDrawingObject *obj=*it;

		// Now move the object
		obj->a.x += r.x;
		obj->a.y += r.y;
		obj->b.x += r.x;
		obj->b.y += r.y;

		// Make a copy of the selected objects (so as to perform add/delete operation for Redo List)
		MarkChangeForUndo( obj );

		// Now move the object back...
		obj->a.x -= r.x;
		obj->a.y -= r.y;
		obj->b.x -= r.x;
		obj->b.y -= r.y;

		++ it;
	}


  SetModifiedFlag( TRUE );

}









// Load a new design
BOOL CTinyCadDoc::Import()
{

  OPENFILENAME ofn;
  char szFile[256], szFileTitle[256];

  strcpy(szFile, "*.dsn\0");

  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner = AfxGetMainWnd()->m_hWnd;
  ofn.lpstrFilter = "Designs (*.dsn)\0*.dsn\0All files (*.*)\0*.*\0\0";
  ofn.lpstrCustomFilter = NULL;
  ofn.nMaxCustFilter = 0;
  ofn.nFilterIndex = 1;
  ofn.lpstrFile = szFile;
  ofn.nMaxFile = sizeof(szFile);
  ofn.lpstrFileTitle = szFileTitle;
  ofn.nMaxFileTitle = sizeof(szFileTitle);
  ofn.lpstrInitialDir = NULL;
  ofn.lpstrTitle = NULL;
  ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
  ofn.nFileOffset = 0;
  ofn.nFileExtension = 0;
  ofn.lpstrDefExt = "dsn";
  ofn.lpstrTitle = "Import";
 
  if (!GetOpenFileName(&ofn))
       	return FALSE;

  // Do we use the text loading procedure or load from a CArchive?
	CFile theFile;
	if (!theFile.Open(szFile, CFile::modeRead)) 
	{
		// Could not open file to start loading
		Message(IDS_ABORTLOAD,MB_ICONEXCLAMATION);
		return FALSE;
	}

	CStreamFile stream( &theFile, CArchive::load );
  	return Import( stream );
}



void CTinyCadDoc::ForceSetCursor()
{
	m_pParent->UpdateAllViews( NULL, DOC_UPDATE_SETCURSOR, NULL );
}


// Redraw our window
void CTinyCadDoc::Invalidate()
{
	if (m_pParent)
	{
		m_pParent->UpdateAllViews( NULL, DOC_UPDATE_INVALIDATE, NULL );
	}
}

void CTinyCadDoc::InvalidateRect( CRect r, BOOL erase, int grow, BOOL outline_only )
{
	if (outline_only)
	{
		CRect r1( r.left, r.top, r.right, r.top );
		CRect r2( r.right, r.top, r.right, r.bottom );
		CRect r3( r.right, r.bottom, r.left, r.bottom );
		CRect r4( r.left, r.bottom, r.left, r.top );
		InvalidateRect( r1, erase, grow );
		InvalidateRect( r2, erase, grow );
		InvalidateRect( r3, erase, grow );
		InvalidateRect( r4, erase, grow );
	}
	else
	{
		doc_invalidrect hint;
		hint.r = r;
		hint.grow = grow;
		if (m_pParent)
		{
			m_pParent->UpdateAllViews( NULL, erase ? DOC_UPDATE_INVALIDRECTERASE : DOC_UPDATE_INVALIDRECT, &hint );
		}
	}
}

// No obvious answer to this one, but it is
// a function that should go in here...
CSize CTinyCadDoc::GetTextExtent(const char *str, int style )
{
	Transform dummy;
	Context dc(AfxGetMainWnd(), dummy );
	dc.SelectFont(*GetOptions()->GetFont(style),0);
	return dc.GetTextExtent( str );
}

// Select/add a new symbol for editing
void CTinyCadDoc::SelectSymbol( CLibraryStoreSymbol *theSymbol )
{
	SelectObject(new CDrawEditItem(this));
	hSYMBOL symbol = GetOptions()->AddSymbol( theSymbol->GetDesignSymbol(this) );
	CDrawMethod *NewMethod = new CDrawMethod(this, symbol, theSymbol->m_pParent->orientation );

	SelectObject(NewMethod);
}


// Select a new editing object
void CTinyCadDoc::SelectObject(CDrawingObject *NewO )
{ 
	if (edit)
	{
		edit->EndEdit();
		delete edit;
	}

	if (NewO)
	{
		if (NewO->GetType() != xEditItem)
		{
			BeginNewChangeSet();
		}
		NewO->BeginEdit(FALSE);
	}

  edit=NewO;
}

// Draw the design rulers and the details box
void CTinyCadDoc::Display( Context& dc )
{
	GetDetails().Display( dc, theOptions, m_pParent->GetPathName() );
}

void CTinyCadDoc::SetSelectable(CDrawingObject *obj)
{
	// Only redraw if the selectable changes
	if (obj != selectable)
	{
		if (selectable)
		{
			selectable->Display( FALSE );
		}

		selectable = obj;

		if (selectable)
		{
			selectable->Display( FALSE );
		}
	}
}

CDrawingObject* CTinyCadDoc::GetSelectable()
{
	return selectable;
}

#if 0
BOOL CTinyCadDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	// Is this a emf file?
	CString path = lpszPathName;
	CString extension;
	int brk = path.ReverseFind('.');
	if (brk != -1)
	{
		extension = path.Mid(brk);
	}
	if (extension.CompareNoCase(".emf") == 0)
	{
		// Load a wmf
		HENHMETAFILE data = (HENHMETAFILE)::GetEnhMetaFile( lpszPathName );
		CDrawMetaFile *pObject = new CDrawMetaFile( this );
		pObject->setMetaFile( data );
		Add( pObject );
	}
	else
	{
		// Load normally
		if (!CDocument::OnOpenDocument(lpszPathName))
			return FALSE;
	}
	
	return TRUE;
}
#endif


// Called after a paste or import to enable the
// document to sort out the imported block when
// necessary
void CTinyCadDoc::PostPaste()
{
}



CPoint CTinyCadDoc::GetStickyPoint( CPoint q, BOOL pins, BOOL wires, BOOL &is_stuck, BOOL &is_junction )
{
  CPoint r(0,0);
  bool first = true;
  double min_distance = 0;
  int items = 0;

  int range = GetOptions()->GetAutoSnapRange();

  if (!GetOptions()->GetAutoSnap())
  {
  	  is_stuck = FALSE;
	  is_junction = FALSE;
	  return q;
  }


  // Search for methods, and look at their pins
	drawingIterator it = GetDrawingBegin();
	while (it != GetDrawingEnd()) 
	{
		CDrawingObject *ObjPtr = *it;

		int hold;
		CPoint tr;

		switch (ObjPtr->GetType()) 
		{
			case xWire:
				#define theLine ((CDrawLine*)ObjPtr)
				if (wires)
				{
					CPoint d;
					CLineUtils l( theLine->a, theLine->b );
					double distance = l.DistanceFromPoint( q, d );

					if (d == r)
					{
						items ++;
					}

					if (first || distance < min_distance)
					{
						if (r != d)
						{
							// have we split this wire?
							if (d != theLine->a && d != theLine->b)
							{
								items = 2;
							}
							else
							{
								items = 1;
							}
						}
						r = d;
						first = false;
						min_distance = distance;
					}
				}
			break;
			case xMethodEx3:
				if (pins)
				{

					CRect s( ObjPtr->a.x, ObjPtr->a.y, ObjPtr->b.x, ObjPtr->b.y );
					s.NormalizeRect();
					s.left -= range * 2;
					s.right += range * 2;
					s.bottom += range * 2;
					s.top -= range * 2;

					if (s.PtInRect(q))
					{
					
						CDrawMethod *theMethod = static_cast<CDrawMethod*>(ObjPtr);
						hold = GetPart();
						SetPart(theMethod->GetSubPart());

						drawingCollection method;
						((CDrawMethod *)ObjPtr)->ExtractSymbol(tr,method);

						drawingIterator it = method.begin();
						while (it != method.end()) 
						{
							CDrawingObject *pointer = *it;

							CDrawPin *thePin = static_cast<CDrawPin*>(pointer);
							if (pointer->GetType()==xPinEx && !(thePin->IsPower()) && !(thePin->IsInvisible()))
							{
								// This is a valid pin...
								CPoint d = thePin->GetPosition(theMethod);
								double dx = d.x - q.x;
								double dy = d.y - q.y;
								double distance = sqrt(dx*dx + dy*dy);

								if (r == d)
								{
									items ++;
								}

								if (first || distance < min_distance)
								{
									if (r != d)
									{
										items = 1;
									}

									r = d;
									first = false;
									min_distance = distance;
								}
							}

							++ it;
						}
						SetPart(hold);
					}
				}
			break;
			case xBusSlash:
				if (pins)
				{
					CPoint d1 = ObjPtr->a;
					double d1x = d1.x - q.x;
					double d1y = d1.y - q.y;
					double distance1 = sqrt(d1x*d1x + d1y*d1y);

					CPoint d2 = ObjPtr->b;
					double d2x = d2.x - q.x;
					double d2y = d2.y - q.y;
					double distance2 = d2x*d2x + d2y*d2y;

					// Which point to use?
					if (distance2 < distance1)
					{
						distance1 = distance2;
						d1 = d2;
					}

					if (r == d1)
					{
						items ++;
					}

					if (first || distance1 < min_distance)
					{
						if (r != d1)
						{
							items = 1;
						}

						r = d1;
						first = false;
						min_distance = distance1;
					}
				}
				break;
			case xPower:
				if (pins)
				{
					// This is a valid pin...
					CPoint d = ObjPtr->a;
					double dx = d.x - q.x;
					double dy = d.y - q.y;
					double distance = sqrt(dx*dx + dy*dy);

					if (r == d)
					{
						items ++;
					}

					if (first || distance < min_distance)
					{
						if (r != d)
						{
							items = 1;
						}

						r = d;
						first = false;
						min_distance = distance;
					}
				}
			}

		++ it;
	}



  if (!first && min_distance < GetOptions()->GetAutoSnapRange())
  {
	  is_stuck = TRUE;
	  is_junction = GetOptions()->GetAutoJunc() ? items > 1 : FALSE;
	  return r;
  }
  else
  {
	  is_junction = FALSE;
	  is_stuck = FALSE;
  }

  return q;
}

// Duplicate the selected objects
void CTinyCadDoc::SelectDup()
{
	drawingCollection newSelection;

	selectIterator it = GetSelectBegin();
	while ( it != GetSelectEnd() ) 
	{
		CDrawingObject *obj=*it;

		CDrawingObject *pNewObject = obj->Store();
		newSelection.push_back( pNewObject );

		obj->Display();
		++ it;
	}



	// Swap over the selection to the new
	// objects...
	UnSelect();
	drawingCollection::iterator itx = newSelection.begin();
	while ( itx != newSelection.end() ) 
	{
		Select( *itx );

		// ... and mark for change...
		MarkChangeForUndo( *itx );
		++ itx;
	}
}


// Move the selected items (the CPoint is the relative shift)
void CTinyCadDoc::SelectMove(CPoint r)
{
  CJunctionUtils j( this );

  if (!IsSelected())
	return;


	// Remove all the objects from the screen
  	selectIterator it = GetSelectBegin();
	while ( it != GetSelectEnd() ) 
	{
		CDrawingObject *obj=*it;

 		// Redraw it's old position
		obj->Display();

		// Add this object to the todo list..
		j.AddObjectToTodo( obj );

		// Move the object
		obj->a.x += r.x;
		obj->a.y += r.y;
		obj->b.x += r.x;
		obj->b.y += r.y;

		// Add the new position to the todo list
		j.AddObjectToTodo( obj );

		obj->Display();

		++ it;
  }
  
  j.CheckTodoList( false );
}


void CTinyCadDoc::AddNewJunction( CPoint q )
{
	// A junction is required, create it!
	CDrawJunction *junction = new CDrawJunction( this );
	junction->a = q;
	junction->b = q;

	m_drawing.insert( m_drawing.end(), junction );
	MarkAdditionForUndo( junction );
	junction->Display();
}
//-------------------------------------------------------------------------
CDetails& CTinyCadDoc::GetDetails()
{
	return m_oDetails;
}
//-------------------------------------------------------------------------

// Get the current options for this document
COption *CTinyCadDoc::GetOptions()
{ 
	return &theOptions; 
}

void CTinyCadDoc::SetModifiedFlag(BOOL Changed )
{
	if (m_pParent)
	{
		m_pParent->SetModifiedFlag( Changed );
	}
}

//-------------------------------------------------------------------------
CString	CTinyCadDoc::GetSheetName()
{
	return m_sheet_name;
}

//-------------------------------------------------------------------------
void CTinyCadDoc::SetSheetName( const char *n )
{
	m_sheet_name = n;
}
