/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002,2003 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "revision.h"
#include "registry.h"
#include "option.h"
#include "context.h"
#include "registry.h"
#include "colour.h"
#include "math.h"
#include "registry.h"


// The constructor
TransformSnap::TransformSnap()
{
  SetAccurateGrid(NormalGrid);
  SetGridSnap(TRUE);
}


// Snap an int to the grid
int TransformSnap::Snap(LONG a)
{
	if (!GridSnap || grid == 0)
		return a;                                 

	// Perform operations at 10 times point, to enable higher accuracy
	a *= 10;
	int Mod = a % grid;
	a = (a - Mod);
	if (Mod * 2 > grid)
		a += grid;
	return a / 10;
}




////// The Scaling operation for translation, rotation and mirroring //////

// The Constructor
Transform::Transform()
{
  SetOriginX(-1);
  SetOriginY(-1);
  SetZoom(256);
  rotmir = 0;

  IsYUp = FALSE;
  BoundY = 0;

  scaling_x = 1.0;
  scaling_y = 1.0;
  pixel_offset = CPoint(0,0);
}


CPoint Transform::SetTRM(CPoint os,CPoint bound,int rotate)
{
  CPoint old=CPoint(m_x,m_y);

  m_x-=os.x;
  m_y-=os.y;
  rotmir=rotate;

  // Calculate the offset given the rotation and bounding box
  switch (rotmir & 3) {
	case 1:		// Down
		m_y-=bound.y ;
		break;
	case 3:		// Right
		m_x-=bound.y ;
		break;
  }
  if ((rotmir &4)!=0) {
	switch (rotmir & 3) {
		case 0:		// Up
		case 1:		// Down
			m_x-=bound.x ;
			break;
		case 2:		// Left
		case 3:		// Right
			m_y-=bound.x ;
			break;
	}
  }


  return old;
}

void Transform::SetScaling( int rot, double x, double y ) 
{ 

  switch (rot & 3)
  {
    case 0:		// Up
	case 1:		// Down
		scaling_x = x; 
		scaling_y = y; 
		break;
	case 2:		// Left
	case 3:		// Right
		scaling_x = y; 
		scaling_y = x; 
		break;
  }
}

void Transform::GetScaling( int rot, double &x, double &y ) 
{ 
  switch (rot & 3)
  {
    case 0:		// Up
	case 1:		// Down
		x = scaling_x; 
		y = scaling_y; 
		break;
	case 2:		// Left
	case 3:		// Right
		y = scaling_x; 
		x = scaling_y; 
		break;
  }
}


void Transform::EndTRM(CPoint os)
{
  rotmir=0;
  m_x=os.x;
  m_y=os.y;
}


// Scale with rotation and mirroring
CPoint Transform::Scale(CPoint p)
{

  // Do we mirror?
  if ((rotmir &4)!=0) {
	p=CPoint(-p.x,p.y);
  }

  switch (rotmir & 3) {
	case 1: 	// Down
		p=CPoint(p.x,-p.y);
		break;
	case 2:		// Left
		p=CPoint(p.y,p.x);
		break;
	case 3:		// Right
		p=CPoint(-p.y,p.x);
		break;
  }
    
  return CPoint(ScaleX(p.x),ScaleY(p.y));
}


// Scale with rotation and mirroring
CPoint Transform::Scale(CDPoint p)
{

  // Do we mirror?
  if ((rotmir &4)!=0) 
  {
	p=CDPoint(-p.x,p.y);
  }

  switch (rotmir & 3) {
	case 1: 	// Down
		p=CDPoint(p.x,-p.y);
		break;
	case 2:		// Left
		p=CDPoint(p.y,p.x);
		break;
	case 3:		// Right
		p=CDPoint(-p.y,p.x);
		break;
  }
    
  return CPoint(ScaleX(p.x)+0.5,ScaleY(p.y)+0.5);
}



int Transform::intScale(LONG a)
{
	return ((a*zoom)>>8);
}

double Transform::doubleScale(double a)
{
	return ((a*zoom)/256.0);
}

int Transform::intDeScale(LONG a)
{
	return ((a<<8)/zoom);
}



// Rotate a direction (for power, pins etc)
int Transform::RotateDir(int in)
{
  
  // Current rotation=> 0  1  2  3
  // Current dir ..						  \/..
  int table[] = {		0, 1, 2, 3, 	// 0
						1, 0, 3, 2,		// 1
						2, 2, 0, 0,		// 2
						3, 3, 1, 1,		// 3
		};

  // Mirror if necessary
  if ((rotmir&4)!=0) {
	if (in==3)
		in=2;
	else if (in==2)
		in=3;
  }

  // Now rotate
  in = table[(rotmir&3)+(in&3)*4];

  return in;
}


